local IMG =  "Interface\\AddOns\\PowerBars\\imgs\\";
local _, Class = UnitClass("Player");
local MsgLog = PowerBars.MsgLog;
local CurrFrame = PowerBars.CurrFrame;
local doDruidUnlock = PowerBars.doDruidUnlock;
local doRegUnlock = PowerBars.doRegUnlock;

function PowerBars:doMoveAndRescale(doFrame)
	local Timer = CreateFrame("FRAME");
	Timer:Hide();

	doFrame:SetScript("OnDragStart", 
		function(self)
			parent = self:GetParent();
			if parent.doMove and not parent.isMoving then
				parent:StartMoving();
				parent:ClearAllPoints();
				parent.isMoving = true;
			end
		end);
		
	doFrame:SetScript("OnDragStop",
		function(self)
			parent = self:GetParent();
			if parent.doMove and parent.isMoving then
				parent:StopMovingOrSizing();
				local FrameSet = parent.Settings;
				FrameSet.Point, _, FrameSet.RelativePoint, FrameSet.X_Ofs, FrameSet.Y_Ofs = parent:GetPoint();
				parent.Settings.isMoved = 1;
				parent.isMoving = nil;
			end
		end);
		
	doFrame:SetScript("OnMouseWheel", 
		function(self, delta)
			parent = self:GetParent();
			if parent.doScaling then
				parent:StartMoving();
				local Scale = parent:GetScale();
				if delta == 1 then
					Scale = Scale + 0.1;
				elseif delta == -1 then
					if Scale > 0.1 then
						Scale = Scale - 0.1;
					end
				end
				if Scale ~= 1 then
					parent.Settings.isScaled = 1;
				else
					parent.Settings.isScaled = 0;
				end
				parent:SetScale(Scale);
				parent.Settings.Scale = Scale;
				parent:StopMovingOrSizing();
				parent:ClearAllPoints();
				parent:SetPoint(parent.Settings.Point, UIParent, parent.Settings.RelativePoint, parent.Settings.X_Ofs, parent.Settings.Y_Ofs);
			end
		end);
	

	doFrame:SetScript("OnEnter",
		function(self)
			parent = self:GetParent();
			local Countdown = 1;
			parent.Overlay.Move:SetAlpha(0.75);
			Timer:SetScript("OnUpdate",
			function(self, elapsed)
				Countdown = Countdown-elapsed;
				if Countdown <= 0.99 and IsMouseButtonDown("LeftButton") ~= 1 then
					local NumFrames = {};
					local StrataList = {"TOOLTIP", "DIALOG", "HIGH", "MEDIUM", "LOW"};
					local minSize = 0;
					for i=1,#CurrFrame do
						if CurrFrame[i].doMove then
							if MouseIsOver(CurrFrame[i]) and CurrFrame[i]:IsVisible() == 1 then
								local effSize = (CurrFrame[i]:GetWidth()+CurrFrame[i]:GetHeight())*CurrFrame[i]:GetScale();
								if minSize == 0 then
									minSize = effSize;
								end
								if minSize > effSize then
									minSize = effSize;
									table.insert(NumFrames, 1, i);
								else
									table.insert(NumFrames, (#NumFrames+1), i);
								end
							end
						end
					end
					for i=1,#NumFrames do
						if i > #StrataList then
						CurrFrame[NumFrames[i]]:SetFrameStrata(StrataList[#StrataList])	;
					else
							CurrFrame[NumFrames[i]]:SetFrameStrata(StrataList[i]);
						end
					end
					Countdown = 1;
				end
			end);
			Timer:Show();
		end);
	
	doFrame:SetScript("OnLeave",
		function(self)
			parent = self:GetParent();
			Timer:Hide();
			parent.Overlay.Move:SetAlpha(0.5);
			parent:SetFrameStrata(parent.Strata);
		end);
end

function PowerBars:doRuneFrames()
	local Runes = PowerBars.Runes;
	local Base = PowerBars.Base.Resource.Base;
	local Frame = PowerBarsDB.PowSet.Frame;
	for i=1,6 do
		local Rune = Runes[i];
		local RuneF = PowerBarsDB.PowSet.Frame[i+2];
		Rune.Frame:SetSize(RuneF.Width, RuneF.Height);
		if PowerBarsDB.PowSet.Unlocked == 1 then
			Rune.Frame:SetScale(RuneF.Scale);
		else
			Rune.Frame:SetScale(Frame[1].Scale);
		end
		Rune.Frame:ClearAllPoints();
		if PowerBarsDB.PowSet.Unlocked == 1 then
			Rune.Frame:SetPoint(RuneF.Point, UIParent, RuneF.RelativePoint, RuneF.X_Ofs, RuneF.Y_Ofs);
		else
			if i == 1 then
				Rune.Frame:SetPoint(RuneF.Point);
			elseif i % 2 == 0 then
				Rune.Frame:SetPoint(RuneF.Point, Runes[i-1].Frame, RuneF.RelativePoint);
			else
				if PowerBarsDB.Options.DEATHKNIGHT.Layout == 3 or PowerBarsDB.Options.DEATHKNIGHT.Layout == 4 then
					Rune.Frame:SetPoint(RuneF.Point, Runes[i-1].Frame, RuneF.RelativePoint);
				else
					Rune.Frame:SetPoint(RuneF.Point, Runes[i-2].Frame, RuneF.RelativePoint);
				end
			 end
		end
	end
end

function PowerBars:GetEclipseArrow(Eclipse)
	local Arrow = 0;
	local modVert = 0;
	if PowerBarsDB.Options.DRUID.isVert == 1 then
		modVert = 7;
	end
	if Eclipse > 0 then
		if GetEclipseDirection() == "sun" then
			Arrow = 6;
			else
			Arrow = 4;
		end
	elseif Eclipse < 0 then
		if GetEclipseDirection() == "sun" then
			Arrow = 3;
		else
			Arrow = 5;
		end
	else
		if GetEclipseDirection() == "sun" then
			Arrow = 6;
		else
			Arrow = 5;
		end
	end
	
	return (Arrow+modVert);
end

function PowerBars:doEclipseFrames()
		local Frame = PowerBarsDB.PowSet.Frame;
		local Base = PowerBars.Base.Resource.Base;
		local Eclipse = PowerBars.Eclipse;
		local Bar = Eclipse.Bar;
		Bar.Frame:SetSize(Frame[3].Width, Frame[3].Height);
		Bar.Frame:SetScale(Frame[3].Scale);
		if PowerBarsDB.Options.DRUID.Unlocked == 1 and Frame[3].isMoved == 1 then
			Bar.Frame:SetPoint(Frame[3].Point, UIParent, Frame[3].RelativePoint, Frame[3].X_Ofs, Frame[3].Y_Ofs);
		else
			Bar.Frame:SetPoint(Frame[3].Point, Base, Frame[3].RelativePoint, Frame[3].X_Ofs, Frame[3].Y_Ofs);
		end
		Bar.Move:SetAllPoints(Bar.Frame);
		local Arrow = Eclipse.Arrow;
		Arrow.Frame:SetSize(Frame[4].Width, Frame[4].Height);
		local EclipsePow = UnitPower("Player", 8);
		Arrow.Frame:SetPoint(Frame[4].Point, Bar.Frame, Frame[4].RelativePoint, (PowerBars.Eclipse.Bar.Frame:GetWidth()/200) * EclipsePow, 0);
		local Text = Eclipse.Text;
		Text.Frame:SetSize(Frame[5].Width, Frame[5].Height);
		if PowerBarsDB.Options.DRUID.Unlocked == 1 and Frame[5].isMoved == 1 then
			Text.Frame:SetPoint(Frame[5].Point, UIParent, Frame[5].RelativePoint, Frame[5].X_Ofs, Frame[5].Y_Ofs);
		else
			Text.Frame:SetPoint(Frame[5].Point, Base, Frame[5].RelativePoint, Frame[5].X_Ofs, Frame[5].Y_Ofs);
		end
		Text.Move:SetAllPoints(Text.Frame);
		Text.Font:SetFont("Fonts\\FRIZQT__.TTF", Text.Frame:GetHeight()*0.75, "OUTLINE");
		local vText = Eclipse.vText;
		vText.Frame:SetSize(Frame[6].Width, Frame[6].Height);
		if PowerBarsDB.Options.DRUID.Unlocked == 1 and Frame[6].isMoved == 1 then
			vText.Frame:SetPoint(Frame[6].Point, UIParent, Frame[6].RelativePoint, Frame[6].X_Ofs, Frame[6].Y_Ofs);
		else
			vText.Frame:SetPoint(Frame[6].Point, Arrow.Frame, Frame[6].RelativePoint, Frame[6].X_Ofs, Frame[6].Y_Ofs);
		end
		vText.Move:SetAllPoints(vText.Frame);
		vText.Font:SetFont("Fonts\\FRIZQT__.TTF", vText.Frame:GetHeight()*0.75, "OUTLINE");
		if PowerBarsDB.Options.DRUID.isVert == 1 then
			PowerBars:SetEclipseVert(1);
		end
end

function PowerBars:ChangePow()
	local StatusBar = PowerBars.Base.Power.StatusBar;
	local Text = StatusBar.Overimg.Font;
	local Pow = PowerBars.Pow;
	if select(2, StatusBar:GetMinMaxValues()) ~= UnitPowerMax("Player", Pow[1].Type) then
		StatusBar:SetMinMaxValues(0, UnitPowerMax("Player", Pow[1].Type));
	end
	StatusBar:SetValue(UnitPower("Player", Pow[1].Type));
	Text:SetText(UnitPower("Player", Pow[1].Type).."/"..select(2, StatusBar:GetMinMaxValues()));
end

function PowerBars:ChangePowWarlock()
	local img = IMG..PowerBarsDB.PowSet.PowIMG;
	local Resource = PowerBars.Base.Resource;
	if PowerBarsDB.PowSet.ResTexture == 1 then
		for i=1,3 do
			if UnitBuff("Player", GetSpellInfo(74434)) and PowerBarsDB.Options.WARLOCK.Soulburn == 1 and UnitPower("Player", PowerBars.Pow[2].Type) < i then
				local BaseIMG = img..(i+8);
				PowerBars.Base.Resource.Base.Frame[i].Texture:SetTexture(BaseIMG);
			else
				local BaseIMG = img..(i-1);
				PowerBars.Base.Resource.Base.Frame[i].Texture:SetTexture(BaseIMG);
			end
		end
	end
	local Power = UnitPower("Player", PowerBars.Pow[2].Type);
	local CurrImg = Power + 2;
	local i = 5;
	while i > 2 do
		if i > CurrImg then
			Resource.Base.Frame[i-2].ResTexture:SetTexture("");
		else
			-- Soulburn #(74434)
			if UnitBuff("Player", GetSpellInfo(74434)) and PowerBarsDB.Options.WARLOCK.Soulburn == 1 then
				Resource.Base.Frame[i-2].ResTexture:SetTexture(img..(i+3));
			else
				Resource.Base.Frame[i-2].ResTexture:SetTexture(img..i);
			end
		end
	i = i - 1;
	end
	PowerBars:ChangePow();
end

function PowerBars:ChangePowPaladin(isHPW)
	local img = IMG..PowerBarsDB.PowSet.PowIMG;
	local Resource = PowerBars.Base.Resource;
	local newHPW = UnitPower("Player", 9);
	if PowerBarsDB.PowSet.ResTexture == 1 then
		for i=1,3 do
			local BaseIMG = img..(i-1);
			PowerBars.Base.Resource.Base.Frame[i].Texture:SetTexture(BaseIMG);
		end
	end
	local Power = UnitPower("Player", PowerBars.Pow[2].Type);
	local CurrImg = Power + 2;
	local i = 5;
	while i > 2 do
		if i > CurrImg then
			Resource.Base.Frame[i-2].ResTexture:SetTexture("");
		else
			Resource.Base.Frame[i-2].ResTexture:SetTexture(img..i);
		end
	i = i - 1;
	end
	if PowerBarsDB.Options.PALADIN.NumHPW > 0 and UnitPower("Player", PowerBars.Pow[2].Type) < PowerBarsDB.Options.PALADIN.NumHPW then
		Resource.Base:Hide();
	else
		if PowerBarsDB.PowSet.ResBase == 1 then
			Resource.Base:Show();
		else 
			Resource.Base:Hide();
		end
	end
	if isHPW then
		if newHPW >= self.currHPW then
			MsgLog:AddLine("Holy Power Gained!");
		else
			local currTime = GetTime();
			local elapsedTime = currTime - self.lastHPW;
			MsgLog:AddLine("Holy Power Lost! Seconds:"..elapsedTime);
		end
		self.lastHPW = GetTime();
	end
	self.currHPW = UnitPower("Player", 9);
	PowerBars:ChangePow();
end

function PowerBars:CalcVirtEclipse()
	local SpellTable = PowerBars.SpellTable;
	local NumItems = #(SpellTable);
	local vEclipse = 0;
	for i=1,NumItems do
		if SpellTable[i].Action ~= "Stopped" then
			vEclipse = vEclipse + SpellTable[i].Energy;
		end
	end
	return vEclipse
end

function PowerBars:ChangePowDruid()
	local PowIMG = PowerBarsDB.PowSet.PowIMG;
	local img = IMG .. PowIMG;
	if GetPrimaryTalentTree() == 1 then
		if PowerBars.CheckIsBaseTrue then
			PowerBars.CheckIsBaseTrue:Disable();
		end
		if PowerBarsDB.PowSet.ResTexture == 1 then
			PowerBarsDB.PowSet.ResTexture = 0;
			PowerBars.Base.Resource.Base.Texture:SetTexture("");
		else
			PowerBars.Base.Resource.Base.Texture:SetTexture("");
		end
		local EclipsePow = UnitPower("Player", 8);
		local vEclipse = EclipsePow + PowerBars:CalcVirtEclipse();
		if vEclipse > 100 then
			vEclipse = 100;
		elseif vEclipse < -100 then
			vEclipse = -100;
		end
		local modVert = 0;
		if PowerBarsDB.Options.DRUID.isVert == 1 then
			modVert = 7;
		end
		 -- Eclipse [Solar] #(48517)
		if UnitBuff("Player", GetSpellInfo(48517)) then
			PowIMG = img .. (1+modVert);
		-- Eclipse [Lunar] #(48518)
		elseif UnitBuff("Player", GetSpellInfo(48518)) then
			PowIMG = img .. (2+modVert);
		else
			PowIMG = img .. (0+modVert);
		end
		if PowerBarsDB.Options.DRUID.vWarnShow == 1 then
			if vEclipse == 100 then
				PowIMG = img .. (1+modVert);
			elseif vEclipse == -100 then
				PowIMG = img .. (2+modVert);
			--[[
			else
				PowIMG = img .. 0;
			--]]
			end
		end
		PowerBars.Base.Resource.Base.ResTexture:SetTexture(PowIMG);
		PowerBars.Eclipse.Arrow.Frame.Texture:SetTexture(img..PowerBars:GetEclipseArrow(EclipsePow));
		if PowerBarsDB.Options.DRUID.isVert == 1 then
			PowerBars.Eclipse.Arrow.Frame:SetPoint("CENTER", PowerBars.Eclipse.Bar.Frame, "CENTER", 0, (PowerBars.Eclipse.Bar.Frame:GetHeight()/200) * EclipsePow);
		else
			PowerBars.Eclipse.Arrow.Frame:SetPoint("CENTER", PowerBars.Eclipse.Bar.Frame, "CENTER", (PowerBars.Eclipse.Bar.Frame:GetWidth()/200) * EclipsePow, 0);
		end
		if PowerBars.Eclipse.Text.Frame.Font then
			local EclipseAbs = EclipsePow;
			if EclipseAbs < 0 then
				EclipseAbs = EclipseAbs * -1;
			end
			PowerBars.Eclipse.Text.Frame.Font:SetText(EclipseAbs);
		end
		if PowerBars.Eclipse.vText.Frame.Font then
			local vEclipseAbs = vEclipse;
			if vEclipseAbs < 0 then
				vEclipseAbs = vEclipseAbs * -1;
			end
			PowerBars.Eclipse.vText.Frame.Font:SetText(vEclipseAbs);
		end
	end
	if PowerBars.Base.Power.StatusBar then
		PowerBars:ChangePow();
	end
end

function PowerBars:RuneTimer(currRune)
	local Countdown = 1;
	local TimerFrame = CreateFrame("FRAME");
	TimerFrame:SetScript("OnUpdate",
		function(self, elapsed)
			Countdown = Countdown - elapsed;
			if Countdown <= 0 then
				local Runes = PowerBars.Runes;
				local Rune = Runes[currRune];
				local Cooldown = Rune.Frame.Cooldown;
				if Cooldown.currCD > 0 then
					local Text = Cooldown.Frame.Text;
					if PowerBarsDB.Options.DEATHKNIGHT.Timers ~= 0 and PowerBarsDB.Options.DEATHKNIGHT.OmniCC == 0 then
						Cooldown.Frame:Show();
					end
					Cooldown.currCD = Cooldown.currCD - 1;
					if Cooldown.currCD == 4 then
						Text:SetTextColor(1,0,0,1);
					end
					Text:SetText(Cooldown.currCD);
					if Cooldown.currCD == 0 then
						Text:SetTextColor(1,1,0,1);
						if PowerBarsDB.Options.DEATHKNIGHT.ShowCD == 0 then
							Rune.Frame.Texture:SetTexture(IMG..PowerBarsDB.PowSet.PowIMG..GetRuneType(currRune));
							Cooldown:SetAlpha(1);
							Cooldown:SetCooldown(0,0);
							Cooldown:SetAlpha(0);
						end
						Cooldown.Frame:Hide();
					end
				end
			Countdown = Countdown + 1;
			end
		end);
end

function PowerBars:ChangePowDK(event)
	local img = IMG..PowerBarsDB.PowSet.PowIMG;
	if event == "RUNE_POWER_UPDATE" or event == "RUNE_TYPE_UPDATE" then
		local Runes = PowerBars.Runes
		local RuneF = PowerBarsDB.PowSet.Frame;
		for i=1,6,1 do
			local Rune = Runes[i];
			local start, dur, runeReady = GetRuneCooldown(i);
			local Cooldown = Rune.Frame.Cooldown;
			local Text = Cooldown.Frame.Text;
			if PowerBarsDB.PowSet.Aspect.Resource == 1 then
				if GetRuneType(i) == 4 then
					PowerBars.Runes[i].Frame:SetSize(RuneF[9].Width, RuneF[9].Height);
				else
					PowerBars.Runes[i].Frame:SetSize(RuneF[i+2].Width, RuneF[i+2].Height);
				end
			end
			if not runeReady and Cooldown.currCD <= 0 then
				Cooldown:SetCooldown(start, dur);
				local selRune = 0;
				if i % 2 == 0 then
					selRune = i-1;
				else
					selRune = i+1;
				end
				if dur then
					Cooldown.currCD = Runes[selRune].Frame.Cooldown.currCD + math.floor(dur);
				end
			else
				if runeReady and Cooldown.currCD > 0 then
					Cooldown.currCD = 0;
					Cooldown.Frame:Hide();
				end
			end
			if not runeReady and PowerBarsDB.Options.DEATHKNIGHT.ShowCD == 0 and GetRuneType(i) then
				Rune.Frame.Texture:SetTexture(img..(GetRuneType(i)+4));
			else
				Rune.Frame.Texture:SetTexture(img..GetRuneType(i));
			end
		end
	else
		if PowerBars.Base.Power.StatusBar then
			PowerBars:ChangePow();
		end
	end
end

function PowerBars:SetRuneFrames()
	local img = IMG..PowerBarsDB.PowSet.PowIMG;
	if not PowerBars.Runes then
		PowerBars.Runes = {}
		local Runes = PowerBars.Runes;
		local Frame = PowerBarsDB.PowSet.Frame;
		local Base = PowerBars.Base.Resource.Base;
		for i=1,6 do
			Runes[i] = {};
			local RuneF = PowerBarsDB.PowSet.Frame[i+2];
			local Rune = Runes[i];
			Rune.Frame = CreateFrame("FRAME", nil, Base);
			CurrFrame[i+2] = Rune.Frame;
			Rune.Frame.Settings = PowerBarsDB.PowSet.Frame[i+2];
			Rune.Frame:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
			Rune.Frame.Strata = Rune.Frame:GetFrameStrata();
			if PowerBarsDB.PowSet.Aspect.Resource == 1 then
				if GetRuneType(i) == 4 then
					Rune.Frame:SetSize(PowerBarsDB.PowSet.Frame[9].Width, PowerBarsDB.PowSet.Frame[9].Height);
				else
					Rune.Frame:SetSize(RuneF.Width, RuneF.Height);
				end
			else
				Rune.Frame:SetSize(RuneF.Width, RuneF.Height);
			end
			if PowerBarsDB.PowSet.Unlocked == 1 then
				Rune.Frame:SetPoint(RuneF.Point, UIParent, RuneF.RelativePoint, RuneF.X_Ofs, RuneF.Y_Ofs);
				Rune.Frame.isMovable = true;
				Rune.Frame.isRescale = true;
				Rune.Frame:SetScale(RuneF.Scale);
				if RuneF.isMoved == 1 then
					Rune.Frame:SetPoint(RuneF.Point, UIParent, RuneF.RelativePoint, RuneF.X_Ofs, RuneF.Y_Ofs);
				else
					if i == 1 then
						Rune.Frame:SetPoint("TOPLEFT", Base);
					else
						if i % 2 == 0 then
							Rune.Frame:SetPoint("LEFT", Runes[i-1].Frame, "RIGHT");
						else
							Rune.Frame:SetPoint("TOP", Runes[i-2].Frame, "BOTTOM");
						end
					end
				end
			else
				if i == 1 then
					Rune.Frame:SetPoint(RuneF.Point);
				elseif i % 2 == 0 then
					Rune.Frame:SetPoint(RuneF.Point, Runes[i-1].Frame, RuneF.RelativePoint);
				else
					if PowerBarsDB.Options.DEATHKNIGHT.Layout == 3 or PowerBarsDB.Options.DEATHKNIGHT.Layout == 4 then
						Rune.Frame:SetPoint(RuneF.Point, Runes[i-1].Frame, RuneF.RelativePoint);
					else
						Rune.Frame:SetPoint(RuneF.Point, Runes[i-2].Frame, RuneF.RelativePoint);
					end
				end
				Rune.Frame.isMovable = false;
				Rune.Frame.isRescale = false;
				Rune.Frame:SetScale(Frame[1].Scale);
			end
			if i == 1 then
				Rune.Frame.relativeTo = PowerBars.Base.Resource.Base;
			elseif i % 2 == 0 then
				Rune.Frame.relativeTo = Runes[i-1].Frame;
			else
				Rune.Frame.relativeTo = Runes[i-2].Frame;
			end
			Rune.Frame.Texture = Rune.Frame:CreateTexture(nil, "BACKGROUND");
			local Texture = Rune.Frame.Texture
			Texture:SetAllPoints(Rune.Frame);
			--Texture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
			Rune.Frame.Overlay = CreateFrame("BUTTON", nil, Rune.Frame);
			Rune.Frame.Overlay:SetAllPoints(Rune.Frame);
			Rune.Frame.Overlay:Hide()
			Rune.Frame.Overlay.Move = Rune.Frame.Overlay:CreateTexture(nil, "OVERLAY");
			Rune.Frame.Overlay.Move:SetTexture(255/255,255/255,0/255,0.5);
			Rune.Frame.Overlay.Move:SetAllPoints(Rune.Frame.Overlay);
			local currRune = GetRuneType(i) or 0;
			if not GetRuneType(i) then
				if i < 3 then
					currRune = 1;
				end
				if i < 5 and i > 2 then
					currRune = 2;
				end
				if i < 7 and i > 4 then
					currRune = 3;
				end
			end
			Texture:SetTexture(img..currRune);
			Rune.Frame.Cooldown = CreateFrame("Cooldown", "PB_RuneCD"..i, Rune.Frame);
			local Cooldown = Rune.Frame.Cooldown;
			Cooldown:SetAllPoints(Rune.Frame);
			--Cooldown:SetFrameStrata("LOW");
			if IsAddOnLoaded("OmniCC") or IsAddOnLoaded("TullaCooldownCount") then
				PowerBars.Class.DEATHKNIGHT.Check.OmniCC:Enable();
				if PowerBarsDB.Options.DEATHKNIGHT.OmniCC == 1 then
					Cooldown.noCooldownCount = false;
				else
					Cooldown.noCooldownCount = true;
				end
			else
				PowerBars.Class.DEATHKNIGHT.Check.OmniCC:Disable();
			end
			if PowerBarsDB.Options.DEATHKNIGHT.ShowCD == 0 then
				Cooldown:SetAlpha(0);
			else
				Cooldown:SetAlpha(1);
			end
			Cooldown.currCD = 0;
			Cooldown.Frame = CreateFrame("FRAME", nil, Rune.Frame);
			local CDFrame = Cooldown.Frame;
			CDFrame:SetAllPoints(Rune.Frame);
			--CDFrame:SetFrameStrata("MEDIUM");
			CDFrame.Text = PowerBars:CreateFont(CDFrame, Cooldown.currCD, "DEFAULT", Cooldown:GetWidth()/2, 0, 0, "CENTER", {255,255,0,1}, "DRAWINGLAYER", "OUTLINE");
			local Text = CDFrame.Text;
			Text:SetAllPoints(CDFrame);
			CDFrame:Hide();
			Rune.Timer = PowerBars:RuneTimer(i);
		end
	end
end

-- ============================================
--              Foresee Energy
-- Original concept by: Copialinex on WoWInterface
--      in the Balance Power Tracker AddOn
-- ============================================

function PowerBars:AddSpellCast(spell)
	local SpellTable = PowerBars.SpellTable;
	local NumQueue = #(SpellTable) or 0;
	local Starsurge, Starfire, Wrath = GetSpellInfo(78674), GetSpellInfo(2912), GetSpellInfo(5176);
	if spell == Starsurge or spell == Starfire or spell == Wrath then
		PowerBars.CastID = PowerBars.CastID + 1;
		local CurrSpell = NumQueue + 1;
		if NumQueue < 4 then
			-- Do nothing
		elseif NumQueue == 4 then
			CurrSpell = 4;
		else
			PowerBars:CallError("PowerBars.SpellTable has exceeded size limits. Readjusting.");
			while NumQueue > 4 do
				SpellTable[NumQueue] = nil;
				NumQueue = #(SpellTable);
			end
			CurrSpell = 4;
		end
		SpellTable[CurrSpell] = {};
		if spell == Starsurge then
			if GetEclipseDirection() == "sun" then
				SpellTable[CurrSpell].Energy = 15;
			else
				SpellTable[CurrSpell].Energy = -15;
			end
		elseif spell == Starfire and GetEclipseDirection() == "sun" then
			SpellTable[CurrSpell].Energy = 20;
		elseif spell == Wrath and GetEclipseDirection() == "moon" then
			if PowerBars.WrathCount < 2 then
				SpellTable[CurrSpell].Energy = -13;
			else
				SpellTable[CurrSpell].Energy = -14;
			end
		else
			SpellTable[CurrSpell].Energy = 0;
		end
		SpellTable[CurrSpell].Spell = spell;
		SpellTable[CurrSpell].Action = "Casting";
		SpellTable[CurrSpell].CastID = PowerBars.CastID;
		SpellTable[CurrSpell].Time = GetTime()*1000; -- Time stored in milliseconds
		MsgLog:AddLine(SpellTable[CurrSpell].Spell.." ("..SpellTable[CurrSpell].CastID..")".." added.");
	end
end

-- Possible to be used at a later point in time.
function PowerBars:CheckTime()
	local SpellTable = PowerBars.SpellTable;
	local currTime = GetTime()*1000 -- Time in milliseconds
	local i = 1;
	while i<#SpellTable do
		if (currTime-SpellTable[i].Time) >= 3000 then
			PowerBars:CollapseTable(SpellTable,i,#SpellTable);
		else
			i = i+1;
		end
	end
end

function PowerBars:ModifyCast(event, spell)
	local SpellTable = PowerBars.SpellTable;
	local NumQueue = #(SpellTable) or 0;
	local Starsurge, Starfire, Wrath = GetSpellInfo(78674), GetSpellInfo(2912), GetSpellInfo(5176);
	if event == "UNIT_SPELLCAST_START" then
		-- Clear Stopped Spells
		local i = 1;
		while i<#SpellTable do
			if SpellTable[i].Action == "Stopped" then
				--PowerBars:CollapseSpellTable(i,NumQueue);
				PowerBars:CollapseTable(SpellTable,i,#SpellTable);
			else
				i = i+1;
			end
		end
		PowerBars.TotalCasts = PowerBars.TotalCasts + 1;
		PowerBars:AddSpellCast(spell);
		PowerBars:ChangePowDruid();
	elseif event == "UNIT_SPELLCAST_SENT" then
		-- Shooting Stars #(93400)
		if spell == Starsurge and PowerBars.ShootingStars.isActive == 1 then
			local _, isCD, _ = GetSpellCooldown(Starsurge);
			MsgLog:AddLine("Attempt to cast Starsurge. Current CD: "..isCD);
			if isCD == 0 then
				PowerBars.noCast = 0;
				local i = 1;
				while i<#SpellTable do
					if SpellTable[i].Action == "Stopped" then
						--PowerBars:CollapseSpellTable(i,NumQueue);
						PowerBars:CollapseTable(SpellTable,i,#SpellTable);
					else
						i = i+1;
					end
				end
				PowerBars.ShootingStars.isActive = 0;
				PowerBars.TotalCasts = PowerBars.TotalCasts + 1;
				PowerBars:AddSpellCast(spell);
				MsgLog:AddLine("Starsurge ("..SpellTable[#SpellTable].CastID..") ["..SpellTable[#SpellTable].Action.."] added! (Shooting Stars)");
			end
		end
	elseif event == "UNIT_SPELLCAST_STOP" then
		for i=1,NumQueue do
			if SpellTable[i].Action == "Casting" then
				SpellTable[i].Action = "Stopped";
				break
			end
		end
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		if spell == Starsurge then
			MsgLog:AddLine("Successful cast: Starsurge");
		end
		for i=1,NumQueue do
			if SpellTable[i].Action == "Casting" or SpellTable[i].Action == "Stopped" then
				SpellTable[i].Action = "Casted";
				MsgLog:AddLine(SpellTable[i].Spell.." ("..SpellTable[i].CastID..")".." set to: Casted");
				break
			end
		end
	end
	PowerBars:ChangePowDruid();
end

function PowerBars:RemoveSpell(subEvent, arg1, arg2, arg3)
	local SpellTable = PowerBars.SpellTable;
	local NumQueue = #(SpellTable) or 0;
	local Starsurge, Starfire, Wrath = GetSpellInfo(78674), GetSpellInfo(2912), GetSpellInfo(5176);
	if subEvent == "SPELL_DAMAGE" then -- arg1 (spellID) arg2 (spellName)
		if arg2 == Starsurge or arg2 == Starfire or arg2 == Wrath then
			if arg1 == Wrath then
				if PowerBars.WrathCount < 3 then
					PowerBars.WrathCount = PowerBars.WrathCount + 1;
				else
					PowerBars.WrathCount = 0;
				end
			end
			if not SpellTable[1] then
				PowerBars:AddSpellCast(arg2);
			end
			local SpellFound = 0;
			--MsgLog:AddLine("Damaging spell: "..arg2, "SpellTable");
			if NumQueue > 1 then
				local i = 1;
				while i<#SpellTable do
					if arg2 == SpellTable[i].Spell and SpellTable[i].Action == "Casted" then
						--PowerBars:CollapseSpellTable(i,NumQueue);
						MsgLog:AddLine("Checking spells: "..SpellTable[i].Spell.." ("..SpellTable[i].CastID..") ["..SpellTable[i].Action.."]");
						PowerBars:CollapseTable(SpellTable,i,#SpellTable);
						SpellFound = 1;
						i = NumQueue + 1;
						break;
					else
						i = i+1;
					end
				end
			else
				if arg2 == SpellTable[1].Spell then
					SpellTable[1] = nil;
					SpellFound = 1;
				end
			end
			if SpellFound == 0 then
				MsgLog:AddLine("Spells in Table: "..#SpellTable, "SpellTable");
				if #SpellTable > 1 then
					for i=1,#SpellTable do
						MsgLog:AddLine(SpellTable[i].Spell.." ("..SpellTable[i].CastID..") ["..SpellTable[i].Action.."]");
					end
				end
				PowerBars:CallError("Damaging spell: ("..arg2..") not in queue.", "noShow");
				while #SpellTable > 0 do
					SpellTable[#SpellTable] = nil;
				end
			end
		end
	elseif subEvent == "SPELL_ENERGIZE" then -- arg1 (energyAmount)
		if arg1 == -14 or arg1 == -27 then
			PowerBars.WrathCount = 0;
		end
	elseif subEvent == "SPELL_MISSED" then -- arg1 (spellID) arg2 (spellName) arg3 (spellMiss)
		if arg2 == Starsurge or arg2 == Starfire or arg2 == Wrath then
			if not SpellTable[1] then
				PowerBars:AddSpellCast(arg2);
			end
			local SpellFound = 0;
			--MsgLog:AddLine("Missed spell: "..arg2, "SpellTable");
			if NumQueue > 1 then
				local i = 1;
				while i<#SpellTable do
					if arg2 == SpellTable[i].Spell and SpellTable[i].Action == "Casted" then
						--PowerBars:CollapseSpellTable(i,NumQueue);
						PowerBars:CollapseTable(SpellTable,i,#SpellTable);
						SpellFound = 1;
						i = NumQueue + 1;
						break
					else
						i = i+1;
					end
				end
			else
				if arg2 == SpellTable[1].Spell then
					SpellTable[1] = nil;
					SpellFound = 1;
				end
			end
			if SpellFound == 0 then
				MsgLog:AddLine("Spells in Table: "..#SpellTable, "SpellTable");
				if #SpellTable > 1 then
					for i=1,#SpellTable do
						MsgLog:AddLine(SpellTable[i].Spell.." ("..SpellTable[i].CastID..") ["..SpellTable[i].Action.."]");
					end
				end
				PowerBars:CallError("Missed spell: ("..arg2..") not in queue.", "noShow");
				while #SpellTable > 0 do
					SpellTable[#SpellTable] = nil;
				end
			end
		end
	end
	PowerBars:ChangePowDruid();
end

function PowerBars:SetDruidSettings()
	if Class == "DRUID" then
		if GetPrimaryTalentTree() ~= 1 then
			if PowerBars.NumPow > 1 then
				PowerBars.Pow[2] = nil
				PowerBars.NumPow = 1;
			end
				PowerBars.Base.Resource.Base:Hide();
				PowerBars.Base.Resource.Base.ResTexture:SetTexture("");
				PowerBars.Check.DisableResource:Disable();
				PowerBars.Check.DisableBlizzResource:Disable();
				PowerBars.Custom.AspectFrame:Hide();
				PowerBars.Check.DefaultIMG:Disable();
				PowerBars.Custom.Edits.SaveResIMG:Hide();
				PowerBars.Check.vTextShow:Disable();
				PowerBars.Check.vWarnShow:Disable();
				PowerBars.Check.DruidUnlock:Disable();
				PowerBars.Check.isVertical:Disable();
				PowerBars.Check.IndivUnlock:Disable();
				PowerBars.Class.DRUID.Balance:Hide();
		else
			PowerBars.Pow[2] = {};
			PowerBars.Pow[2].Name = "Eclipse";
			PowerBars.Pow[2].Type = 8;
			PowerBars.NumPow = 2;
			PowerBars:SetEclipseFrames();
			PowerBars.Check.DisableResource:Enable();
			PowerBars.Check.DisableBlizzResource:Enable();
			PowerBars.Custom.AspectFrame:Show();
			PowerBars.Check.DefaultIMG:Enable();
			PowerBars.Custom.Edits.SaveResIMG:Show();
			PowerBars.Check.vTextShow:Enable();
			PowerBars.Check.vWarnShow:Enable();
			PowerBars.Check.DruidUnlock:Enable();
			PowerBars.Check.isVertical:Enable();
			PowerBars.Check.IndivUnlock:Enable();
			PowerBars.Class.DRUID.Balance:Show();
		end
	end
end

function PowerBars:SetEclipseVert(isVert)
	local Frame = PowerBarsDB.PowSet.Frame;
	local DEFS = PowerBars.Defaults[Class].Settings;
	if isVert == 1 then
		for i=1,4 do
			if i ~= 2 then
				if i == 1 then
					CurrFrame[i]:SetSize(Frame[i].Height, Frame[i].Width);
				end
			end
		end
		if PowerBarsDB.PowSet.Frame[6].isMoved == 0 and PowerBars.isRotated == 0 then
			PowerBars.Eclipse.vText.Frame:ClearAllPoints();
			PowerBars.Eclipse.vText.Frame:SetPoint("RIGHT", PowerBars.Eclipse.Arrow.Frame, "LEFT");
		end
		if PowerBarsDB.PowSet.Frame[3].isMoved == 0 and PowerBars.isRotated == 0 then
			PowerBars.Eclipse.Bar.Frame:ClearAllPoints();
			PowerBars.Eclipse.Bar.Frame:SetPoint(Frame[3].Point, PowerBars.Base.Resource.Base, Frame[3].RelativePoint, Frame[1].Height*0.03,(Frame[1].Width*0.03)*-1);
		end
		if PowerBarsDB.PowSet.Frame[5].isMoved == 0 and PowerBars.isRotated == 0 then
			PowerBars.Eclipse.Text.Frame:ClearAllPoints();
			PowerBars.Eclipse.Text.Frame:SetPoint(Frame[5].Point, PowerBars.Base.Resource.Base, Frame[5].RelativePoint, Frame[1].Height*0.06,(Frame[1].Width*0.03)*-1);
		end
		if PowerBars.isRotated == 0 then
			PowerBars:ChangePowDruid()
			PowerBars.isRotated = 1;
		end
	else
		for i=1,4 do
			if i ~= 2 then
				if i == 1 then
					CurrFrame[i]:SetSize(Frame[i].Width, Frame[i].Height);
				end
			end
		end
		if PowerBarsDB.PowSet.Frame[6].isMoved == 0 and PowerBars.isRotated == 1 then
				PowerBars.Eclipse.vText.Frame:ClearAllPoints();
				PowerBars.Eclipse.vText.Frame:SetPoint("TOP", PowerBars.Eclipse.Arrow.Frame, "BOTTOM");
		end
		if PowerBarsDB.PowSet.Frame[3].isMoved == 0 and PowerBars.isRotated == 1 then
			PowerBars.Eclipse.Bar.Frame:ClearAllPoints();
			PowerBars.Eclipse.Bar.Frame:SetPoint(Frame[3].Point, PowerBars.Base.Resource.Base, Frame[3].RelativePoint, Frame[3].X_Ofs, Frame[3].Y_Ofs);
		end
		if PowerBarsDB.PowSet.Frame[5].isMoved == 0 and PowerBars.isRotated == 1 then
			PowerBars.Eclipse.Text.Frame:ClearAllPoints();
			PowerBars.Eclipse.Text.Frame:SetPoint(Frame[5].Point, PowerBars.Base.Resource.Base, Frame[5].RelativePoint, Frame[5].X_Ofs, Frame[5].Y_Ofs);
		end
		if PowerBars.isRotated == 1 then
			PowerBars:ChangePowDruid()
			PowerBars.isRotated = 0;
		end
	end
end

function PowerBars:SetEclipseFrames()
	if not PowerBars.Eclipse then
		local Frame = PowerBarsDB.PowSet.Frame;
		local PowIMG = PowerBarsDB.PowSet.PowIMG;
		local Base = PowerBars.Base.Resource.Base;
		local Defaults = PowerBars.Defaults[Class];
		PowerBars.Eclipse = {};
		local Eclipse = PowerBars.Eclipse;
		Eclipse.Bar = {};
		local Bar = Eclipse.Bar;
		Bar.Frame = CreateFrame("FRAME", nil, Base);
		CurrFrame[3] = Bar.Frame;
		Bar.Frame.Strata = Bar.Frame:GetFrameStrata();
		Bar.Frame:SetSize(Frame[3].Width, Frame[3].Height);
		if PowerBarsDB.Options.DRUID.Unlocked == 1 and Frame[3].isMoved == 1 then
			Bar.Frame:SetPoint(Frame[3].Point, UIParent, Frame[3].RelativePoint, Frame[3].X_Ofs, Frame[3].Y_Ofs);
		else
			Bar.Frame:SetPoint(Frame[3].Point, Base, Frame[3].RelativePoint, Frame[3].X_Ofs, Frame[3].Y_Ofs);
		end
		Bar.Frame.relativeTo = Base;
		Bar.Frame.Settings = Frame[3];
		Bar.Frame.Overlay = CreateFrame("BUTTON", nil, Bar.Frame);
		Bar.Frame.Overlay:SetAllPoints(Bar.Frame);
		Bar.Frame.Overlay:Hide()
		Bar.Frame.Overlay.Move = Bar.Frame.Overlay:CreateTexture(nil, "BACKGROUND");
		Bar.Frame.Overlay.Move:SetAllPoints(Bar.Frame.Overlay);
		Bar.Frame.Overlay.Move:SetTexture(0,1,0,0.5);
		Eclipse.Arrow = {};
		local Arrow = Eclipse.Arrow;
		Arrow.Frame = CreateFrame("FRAME", nil, Base);
		CurrFrame[4] = Arrow.Frame;
		Arrow.Frame.Strata = Arrow.Frame:GetFrameStrata();
		Arrow.Frame:SetSize(Frame[4].Width, Frame[4].Height);
		Arrow.Strata = Arrow.Frame:GetFrameStrata();
		local EclipsePow = UnitPower("Player", 8);
		Arrow.Frame:SetPoint(Frame[4].Point, Bar.Frame, Frame[4].RelativePoint, (Bar.Frame:GetWidth()/200) * EclipsePow);
		Arrow.Frame.relativeTo = Bar.Frame;
		Arrow.Frame.Texture = Arrow.Frame:CreateTexture(nil, "BACKGROUND");
		Arrow.Frame.Texture:SetAllPoints(Arrow.Frame);
		Arrow.Frame.Texture:SetTexture(IMG..PowIMG..PowerBars:GetEclipseArrow(EclipsePow));
		Arrow.Frame.Texture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
		Eclipse.Text = {};
		local Text = Eclipse.Text;
		Text.Frame = CreateFrame("FRAME", nil, Base);
		CurrFrame[5] = Text.Frame;
		Text.Frame.Strata = Text.Frame:GetFrameStrata();
		Text.Frame.Settings = Frame[5];
		Text.Frame:SetSize(Frame[5].Width, Frame[5].Height);
		if PowerBarsDB.Options.DRUID.Unlocked == 1 and Frame[5].isMoved == 1 then
			Text.Frame:SetPoint(Frame[5].Point, UIParent, Frame[5].RelativePoint, Frame[5].X_Ofs, Frame[5].Y_Ofs);
		else
			Text.Frame:SetPoint(Frame[5].Point, Base, Frame[5].RelativePoint, Frame[5].X_Ofs, Frame[5].Y_Ofs);
		end
		Text.Frame.relativeTo = Base;
		Text.Frame.Overlay = CreateFrame("BUTTON", nil, Text.Frame);
		Text.Frame.Overlay:SetAllPoints(Text.Frame);
		Text.Frame.Overlay:Hide()
		Text.Frame.Overlay.Move = Text.Frame.Overlay:CreateTexture(nil, "BACKGROUND");
		Text.Frame.Overlay.Move:SetAllPoints(Text.Frame.Overlay);
		Text.Frame.Overlay.Move:SetTexture(0,1,0,0.5);
		local EclipseAbs = EclipsePow;
		if EclipseAbs < 0 then
			EclipseAbs = EclipseAbs * -1;
		end
		Text.Frame.Font = PowerBars:CreateFont(Text.Frame, EclipseAbs, "DEFAULT", Text.Frame:GetHeight()*0.75, 0, 0, "CENTER", {255,255,255,1}, "DRAWLAYER", "OUTLINE");
		Text.Frame.Font:SetAllPoints(Text.Frame);
		Text.Frame.Font:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
		Eclipse.vText = {};
		local vText = Eclipse.vText;
		vText.Frame = CreateFrame("FRAME", nil, Base);
		CurrFrame[6] = vText.Frame;
		vText.Frame.Strata = vText.Frame:GetFrameStrata();
		vText.Frame.Settings = Frame[6];
		vText.Frame:SetSize(Frame[6].Width, Frame[6].Height);
		if PowerBarsDB.Options.DRUID.Unlocked == 1 and Frame[6].isMoved == 1 then
			vText.Frame:SetPoint(Frame[6].Point, UIParent, Frame[6].RelativePoint, Frame[6].X_Ofs, Frame[6].Y_Ofs);
		else
			vText.Frame:SetPoint(Frame[6].Point, Arrow.Frame, Frame[6].RelativePoint, Frame[6].X_Ofs, Frame[6].Y_Ofs);
		end
		vText.Frame.relativeTo = Arrow.Frame;
		vText.Frame.Overlay = CreateFrame("BUTTON", nil, vText.Frame);
		vText.Frame.Overlay:SetAllPoints(vText.Frame);
		vText.Frame.Overlay:Hide()
		vText.Frame.Overlay.Move = vText.Frame.Overlay:CreateTexture(nil, "BACKGROUND");
		vText.Frame.Overlay.Move:SetAllPoints(vText.Frame.Overlay);
		vText.Frame.Overlay.Move:SetTexture(0,1,0,0.5);
		vText.Frame.Font = PowerBars:CreateFont(vText.Frame, EclipseAbs, "DEFAULT", vText.Frame:GetHeight()*0.75, 0, 0, "CENTER", {255,255,0,1}, "DRAWLAYER", "OUTLINE");
		vText.Frame.Font:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha))
		vText.Frame.Font:SetAllPoints(vText.Frame);
		for i=1,4 do
			if PowerBarsDB.PowSet.Unlocked == 1 and i ~= 2 then
				CurrFrame[i+2].isMovable = true;
			elseif PowerBarsDB.PowSet.Unlocked == 1 and i ~= 2 then
				CurrFrame[i+2].isMovable = false;
			else
				CurrFrame[i+2].isMovable = false;
			end
			CurrFrame[i+2].isRescale = false;
		end
		if PowerBarsDB.Options.DRUID.vTextShow == 0 then
			vText.Frame:Hide();
		end
		if PowerBarsDB.Options.DRUID.isVert == 1 then
			PowerBars:SetEclipseVert(1);
		end
	end
	PowerBars.Base.Resource.Base:Show();
end

function PowerBars:AdjustOOCAlpha(inCombat)
	local Resource = PowerBars.Base.Resource;
	local Power = PowerBars.Base.Power;
	local effectiveResAlpha = 1;
	local effectiveBaseAlpha = 1;
	local effectivePowAlpha = 1;
	if inCombat == 1 then
		effectiveResAlpha = PowerBarsDB.PowSet.ResAlpha;
		effectivePowAlpha = PowerBarsDB.PowSet.PowAlpha;
		effectiveBaseAlpha = PowerBarsDB.PowSet.BaseFrameAlpha;
	else
		effectiveResAlpha = PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha;
		effectivePowAlpha = PowerBarsDB.PowSet.PowAlpha * PowerBarsDB.PowSet.PowOOCAlpha;
		effectiveBaseAlpha = PowerBarsDB.PowSet.BaseFrameAlpha * PowerBarsDB.PowSet.ResOOCAlpha;
	end
	if Class == "PALADIN" or Class == "WARLOCK" then
		for i=1,3 do
			Resource.Base.Frame[i].ResTexture:SetAlpha(effectiveResAlpha);
			Resource.Base.Frame[i].Texture:SetAlpha(effectiveBaseAlpha);
		end
	elseif Class == "DEATHKNIGHT" then
		local Runes = PowerBars.Runes;
		for i=1,6 do
			local Rune = Runes[i];
			local Texture = Rune.Frame.Texture
			--Texture:SetAlpha(effectiveResAlpha);
			Rune.Frame:SetAlpha(effectiveResAlpha);
		end
	elseif Class == "DRUID" and GetPrimaryTalentTree() == 1 then
		Resource.Base.ResTexture:SetAlpha(effectiveResAlpha);
		PowerBars.Eclipse.Arrow.Frame.Texture:SetAlpha(effectiveResAlpha);
		PowerBars.Eclipse.Text.Frame.Font:SetAlpha(effectiveResAlpha);
		PowerBars.Eclipse.vText.Frame.Font:SetAlpha(effectiveResAlpha);
	end
	Power.StatusBar:SetAlpha(effectivePowAlpha);
end

function PowerBars:AdjustFormAlpha(isForm)
	if Class == "DRUID" and GetPrimaryTalentTree() == 1 then
		local Resource = PowerBars.Base.Resource;
		local FormAlpha = 1;
		if isForm ~= 1 then
			FormAlpha = PowerBarsDB.Options.DRUID.FormAlpha;
		end
		Resource.Base.ResTexture:SetAlpha(FormAlpha);
 		PowerBars.Eclipse.Arrow.Frame.Texture:SetAlpha(FormAlpha);
		PowerBars.Eclipse.Text.Frame.Font:SetAlpha(FormAlpha);
		PowerBars.Eclipse.vText.Frame.Font:SetAlpha(FormAlpha);
	end
end

function PowerBars:doLock()
	PowerBars.isUnlocked = false;
	for i=1,#CurrFrame do
		if CurrFrame[i].doMove then
			if CurrFrame[i].doScaling then
				CurrFrame[i].doScaling = false;
			end
			CurrFrame[i].Overlay:Hide();
			CurrFrame[i].doMove = false;
			CurrFrame[i]:EnableMouse(false);
			CurrFrame[i]:SetMovable(false);
			--MsgLog:AddLine("Frame: "..i.." locked!");
		end
	end
end

function PowerBars:doUnlock()
	PowerBars.isUnlocked = true;
	for i=1,#CurrFrame do
		if CurrFrame[i].isMovable then
			if CurrFrame[i].isRescale then
				CurrFrame[i].doScaling = true;
			end
			CurrFrame[i]:ClearAllPoints();
			CurrFrame[i].Overlay:Show();
			CurrFrame[i].Overlay.Move:SetAlpha(0.5);
			CurrFrame[i].doMove = true;
			CurrFrame[i]:EnableMouse(true);
			CurrFrame[i]:SetMovable(true);
			CurrFrame[i].Overlay:RegisterForDrag("LeftButton");
			CurrFrame[i].Overlay:RegisterForClicks("LeftButtonUp");
			PowerBars:doMoveAndRescale(CurrFrame[i].Overlay);
			--MsgLog:AddLine("Frame: "..i.." unlocked!");
		end
	end
end

function PowerBars:ShowUnlock()
	-- If the Unlock Dialog doesn't exist, make one
	if not PowerBars.Dialogs.Unlock then
		PowerBars.Dialogs.Unlock = {};
		local Unlock = PowerBars.Dialogs.Unlock;
		
		-- Base Frame for all the Dialog components
		Unlock.Frame = CreateFrame("FRAME", nil, UIParent);
		Unlock.Frame:SetFrameStrata("DIALOG");
		Unlock.Frame:SetToplevel(true);
		Unlock.Frame:EnableMouse(true);
		Unlock.Frame:SetSize(400,260);
		Unlock.Frame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
			tile = true,
			insets = {left = 11, right = 12, top = 12, bottom = 11},
			tileSize = 32,
			edgeSize = 32,	
		});
		Unlock.Frame:SetPoint("TOP", UIParent, "TOP", 0, -30);
		Unlock.Frame:Hide();
		
		-- Title region for our title
		Unlock.TitleRegion = Unlock.Frame:CreateTitleRegion();
		Unlock.TitleRegion:SetAllPoints(Unlock.Frame);
		
		-- Creates the little header area for "Power Bars"
		Unlock.Header = Unlock.Frame:CreateTexture(nil, "ARTWORK");
		Unlock.Header:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header");
		Unlock.Header:SetSize(256,64);
		Unlock.Header:SetPoint("CENTER", Unlock.Frame, "TOP", 0, -20);
		
		-- "Power Bars" title to identify who the dialog belongs to
		Unlock.Title = PowerBars:CreateFont(Unlock.Frame, "Power Bars", "DEFAULT", 14);
		Unlock.Title:ClearAllPoints();
		Unlock.Title:SetPoint("CENTER", Unlock.Header, "CENTER", 0, 10);
		
		-- Instruction text on how to use the buttons in the dialog
		Unlock.Text = PowerBars:CreateFont(Unlock.Frame, "Reset: Resets all currently unlocked frames to their default positions as set by Power Bars.\n\nLock: Locks all currently unlocked frames to their current position.\n\nYou may move an unlocked frame by clicking on it and dragging. Frames can be rescaled by hovering your mouse over the frame and spinning the scrollwheel: Up for larger, down for smaller.\n\n|cFFFFFF00Yellow|cFFFFFFFF frames can be moved and rescaled.\n|cFF00FF00Green|cFFFFFFFF frames can only be moved.", "DEFAULT", 12, 363, 0, "LEFT", {255,255,255,1});
		Unlock.Text:ClearAllPoints();
		Unlock.Text:SetPoint("TOPLEFT", Unlock.Frame, "TOPLEFT", 20, -30);
		
		PowerBars.Buttons.Reset = PowerBars:CreateButton(Unlock.Frame, "PBButtons_Reset", "Reset");
		local Reset = PowerBars.Buttons.Reset;
		Reset:ClearAllPoints();
		Reset:SetPoint("TOPLEFT", Unlock.Text, "BOTTOMLEFT", 60, -10);
		
		PowerBars.Buttons.Lock = PowerBars:CreateButton(Unlock.Frame, "PBLockButton", "Lock");
		local Lock = PowerBars.Buttons.Lock;
		Lock:ClearAllPoints();
		Lock:SetPoint("LEFT", Reset, "RIGHT", 0, 0);
		
		if Class == "DRUID" or Class == "WARLOCK" or Class == "PALADIN" or Class == "DEATHKNIGHT" then
			PowerBars.Check.IndivUnlock = PowerBars:CreateCheck(Unlock.Frame, "PBChecks_IndivUnlock", "Unlock individual elements");
			local IndivUnlock = PowerBars.Check.IndivUnlock;
			IndivUnlock:SetHitRectInsets(0,5,0,0);
			IndivUnlock:ClearAllPoints();
			IndivUnlock:SetPoint("TOPLEFT", Reset, "BOTTOMLEFT", 15, -5);
			if Class == "DRUID" then
				IndivUnlock:SetScript("OnClick", doDruidUnlock)
			else
				IndivUnlock:SetScript("OnClick", doRegUnlock)
			end
			if PowerBarsDB.PowSet.Unlocked == 1 then
				IndivUnlock:SetChecked(true);
			else
				IndivUnlock:SetChecked(false);
			end
		else
			Unlock.Frame:SetSize(400,240);
		end
		
		-- Button Scripts
		-- Script for Reset Button
		Reset:SetScript("OnClick",
			function()
				local DEFS = PowerBars.Defaults[Class].Settings;
				local Settings = PowerBarsDB.PowSet.Frame;
				for i=1,#CurrFrame do
					if PowerBarsDB.PowSet.Aspect.Resource == 0 then
						CurrFrame[i]:SetSize(DEFS[i].WIDTH, DEFS[i].HEIGHT);
					end
					if Settings[i].isScaled == 1 then
						MsgLog:AddLine("Current Frame: "..i.." | Scale: "..CurrFrame[i]:GetScale().." | Def Scale: "..DEFS[i].SCALE);
						Settings[i].Scale = DEFS[i].SCALE;
						CurrFrame[i]:SetScale(Settings[i].Scale);
						Settings[i].isScaled = 0;
					end
					if Settings[i].isMoved == 1 then
						Settings[i].Point = DEFS[i].POINT;
						Settings[i].RelativePoint = DEFS[i].RELATIVEPOINT;
						Settings[i].X_Ofs = DEFS[i].X_OFS;
						Settings[i].Y_Ofs = DEFS[i].Y_OFS;
						CurrFrame[i]:ClearAllPoints();
						--CurrFrame[i]:StartMoving();
						CurrFrame[i]:SetPoint(Settings[i].Point, CurrFrame[i].relativeTo, Settings[i].RelativePoint, Settings[i].X_Ofs, Settings[i].Y_Ofs);
						--CurrFrame[i]:StopMovingOrSizing();
						Settings[i].isMoved = 0;
					end
				end
			end);

		-- Script for Lock Button
		Lock:SetScript("OnClick",
			function()
				PowerBars:doLock();
				Unlock.Frame:Hide();
			end);
	else
		PowerBars.Dialogs.Unlock.Frame:Show();
	end
end

function PowerBars:GetInfo()
	PowerBars.Pow = {};
	local Pow = PowerBars.Pow;
	if Class == "PALADIN" then
		Pow[1] = {};
		Pow[1].Name = "Mana";
		Pow[1].Type = 0;
		Pow[2] = {};
		Pow[2].Name = "Holy Power";
		Pow[2].Type = 9;
		PowerBars.NumPow = 2;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "HPW";
		end
	elseif Class == "WARLOCK" then
		Pow[1] = {};
		Pow[1].Name = "Mana";
		Pow[1].Type = 0;
		Pow[2] = {};
		Pow[2].Name = "Soul Shards";
		Pow[2].Type = 7;
		PowerBars.NumPow = 2;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "Shard";
		end
	elseif Class == "DEATHKNIGHT" then
		Pow[1] = {};
		Pow[1].Name = "Runic Power";
		Pow[1].Type = 6;
		Pow[2] = {};
		Pow[2].Name = "Runes";
		Pow[2].Type = 5;
		PowerBars.NumPow = 2;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "Rune";
		end
	elseif Class == "DRUID" then
		Pow[1] = {};
		Pow[1].Name = "Mana";
		Pow[1].Type = 0;
		PowerBars.NumPow = 1;
		if GetPrimaryTalentTree() == 1 then
			PowerBars.Pow[2] = {};
			PowerBars.Pow[2].Name = "Eclipse";
			PowerBars.Pow[2].Type = 8;
			PowerBars.NumPow = 2;
			if PowerBarsDB.PowSet.DefaultIMG then
				PowerBarsDB.PowSet.PowIMG = "Eclipse";
			end
		end
	elseif Class == "ROGUE" then
		Pow[1] = {};
		Pow[1].Name = "Energy";
		Pow[1].Type = 3;
		PowerBars.NumPow = 1;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "";
		end
	elseif Class == "HUNTER" then
		Pow[1] = {};
		Pow[1].Name = "Focus";
		Pow[1].Type = 2;
		PowerBars.NumPow = 1;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "";
		end
	elseif Class == "WARRIOR" then
		Pow[1] = {};
		Pow[1].Name = "Rage";
		Pow[1].Type = 1;
		PowerBars.NumPow = 1;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "";
		end
	elseif Class == "MAGE" then
		Pow[1] = {};
		Pow[1].Name = "Mana";
		Pow[1].Type = 0;
		PowerBars.NumPow = 1;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "";
		end
	elseif Class == "PRIEST" then
		Pow[1] = {};
		Pow[1].Name = "Mana";
		Pow[1].Type = 0;
		PowerBars.NumPow = 1;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "";
		end
	elseif Class == "SHAMAN" then
		Pow[1] = {};
		Pow[1].Name = "Mana";
		Pow[1].Type = 0;
		PowerBars.NumPow = 1;
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			PowerBarsDB.PowSet.PowIMG = "";
		end
	else
		Pow[1] = {}
		Pow[1].Name = "No Resource";
		Pow[1].Type = -1;
		PowerBars.NumPow = 0;
	end
end

function PowerBars:ShowChangePower(event, args)
	local args = args or {};
	if PowerBarsDB.PowSet.ResBase == 1 then
		if Class == "PALADIN" then
			if event == "UNIT_POWER" then
				local isHPW = false;
				if args[1] == "HOLY_POWER" then
					isHPW = true;
				end
				PowerBars:ChangePowPaladin(isHPW);
			end
		elseif Class == "WARLOCK" then
			if event == "UNIT_POWER" or event == "UNIT_AURA" then
				PowerBars:ChangePowWarlock();
			end
		elseif Class == "DRUID" then
			if not PowerBars.Eclipse then
				PowerBars:SetEclipseFrames();
			end
			if event == "UNIT_POWER" or event == "UNIT_AURA" then
				PowerBars:ChangePowDruid();
			end
		elseif Class == "DEATHKNIGHT" then
			if not PowerBars.Runes then
				PowerBars:SetRuneFrames();
			end
			if event == "UNIT_POWER" or event == "RUNE_POWER_UPDATE" or event == 	"RUNE_TYPE_UPDATE" then
				PowerBars:ChangePowDK(event);
			end
		end
	else
		PowerBars.Base.Resource.Base:Hide();
	end
	if PowerBarsDB.PowSet.PowDisable == 0 then
		PowerBars:ChangePow();
	else
		PowerBars.Base.Power.StatusBar:Hide();
	end
end

function PowerBars:CheckBlizzResFrame()
	if Class == "DEATHKNIGHT" then
		self.BlizzResFrame = RuneFrame;
	elseif Class == "PALADIN" then
		self.BlizzResFrame = PaladinPowerBar;
	elseif Class == "WARLOCK" then
		self.BlizzResFrame = ShardBarFrame;
	elseif Class == "DRUID" and GetPrimaryTalentTree() == 1 then
		self.BlizzResFrame = EclipseBarFrame;
	else
		self.BlizzResFrame = CreateFrame("FRAME")
	end
	if not self.BlizzResFrame:IsShown() then
		PowerBars.Check.DisableBlizzResource:Disable()
	end
	if self.BlizzResFrame then
		self.BlizzResFrame:HookScript("OnShow", 
			function(self) 
				if PowerBarsDB.PowSet.BlizzResDisable == 1 then
					self:Hide(); 
				end
			end);
	end
	if PowerBarsDB.PowSet.BlizzResDisable == 1 then
		self.BlizzResFrame:Hide();
	else
		self.BlizzResFrame:Show();
	end
end

function PowerBars:SetClassImg()
	local PowIMG = PowerBarsDB.PowSet.PowIMG;
	local img = IMG .. PowIMG;
	if Class == "PALADIN" or Class == "WARLOCK" then
		local Resource = PowerBars.Base.Resource;
		if PowerBarsDB.PowSet.ResTexture == 1 then
			for i=1,3 do
				local BaseIMG = img..(i-1);
				PowerBars.Base.Resource.Base.Frame[i].Texture:SetTexture(BaseIMG);
			end
		end
		local Power = UnitPower("Player", PowerBars.Pow[2].Type);
		local CurrImg = Power + 2;
		local i = 5;
		while i > 2 do
			if i > CurrImg then
				Resource.Base.Frame[i-2].ResTexture:SetTexture("");
			else
				Resource.Base.Frame[i-2].ResTexture:SetTexture(img..i);
			end
		i = i - 1;
		end
	elseif Class == "DEATHKNIGHT" then
		local Runes = PowerBars.Runes
		for i=1,6,1 do
			local Rune = Runes[i];
			local start, dur, runeReady = GetRuneCooldown(i);
			local Cooldown = Rune.Frame.Cooldown;
			if not runeReady and PowerBarsDB.Options.DEATHKNIGHT.ShowCD == 0 then
				Rune.Frame.Texture:SetTexture(img..(GetRuneType(i)+4));
			else
				Rune.Frame.Texture:SetTexture(img..GetRuneType(i));
			end
		end
	elseif Class == "DRUID" and GetPrimaryTalentTree() == 1 then
		local EclipsePow = UnitPower("Player", 8);
		if UnitBuff("Player", GetSpellInfo(48517)) then -- Eclipse (Solar)
			PowIMG = img .. 1;
		elseif UnitBuff("Player", GetSpellInfo(48518)) then -- Eclipse (Lunar)
			PowIMG = img .. 2;
		else
			PowIMG = img .. 0;
		end
		PowerBars.Base.Resource.Base.ResTexture:SetTexture(PowIMG);
		PowerBars.Eclipse.Arrow.Texture:SetTexture(img..PowerBars:GetEclipseArrow(EclipsePow));
		PowerBars.Eclipse.Arrow.Frame:SetPoint("CENTER", PowerBars.Eclipse.Bar.Frame, "CENTER", (PowerBars.Eclipse.Bar.Frame:GetWidth()/200) * EclipsePow, 0);
		if PowerBars.Eclipse.Text.Font then
			local EclipseAbs = EclipsePow;
			if EclipseAbs < 0 then
				EclipseAbs = EclipseAbs * -1;
			end
			PowerBars.Eclipse.Text.Font:SetText(EclipseAbs);
		end
	end
end

function PowerBars:InitFrames()
	-- ============================
	-- Base Power & Resource Frames
	-- ============================
	-- Base              
	
	PowerBars:CreateResourceFrames();
	
	PowerBars:CreatePowerFrames();
	
 	PowerBars:SetPowerFrames();
	
 	if PowerBarsDB.PowSet.ResBase == 1 then
 		PowerBars.Base.Resource.Base:Show();
		if Class == "WARLOCK" then
			PowerBars:ChangePowWarlock();
		elseif Class == "DEATHKNIGHT" then
			PowerBars:ChangePowDK();
		elseif Class == "DRUID" then
			if not PowerBars.Eclipse then
				PowerBars:SetEclipseFrames();
			end
			PowerBars:ChangePowDruid();
		elseif Class == "PALADIN" then
			PowerBars:ChangePowPaladin();
		end
	else
		PowerBars.Base.Resource.Base:Hide();
	end
	if PowerBarsDB.PowSet.PowDisable == 0 then
		PowerBars:ChangePow();
	else
		PowerBars.Base.Power.StatusBar:Hide();
	end
	
	PowerBars:CreatePanel();

end

function PowerBars:IfShapeshift(class)
	local form = GetShapeshiftFormID();
	if class == "DRUID" then
		if form == 1 then
			PowerBars.Pow[1].Type = 3;
			PowerBars.Pow[1].Name = "Energy";
		elseif form == 5 then
			PowerBars.Pow[1].Type = 1;
			PowerBars.Pow[1].Name = "Rage";
		else
			PowerBars.Pow[1].Type = 0;
			PowerBars.Pow[1].Name = "Mana";
		end
	end
end

function PowerBars:CreateResourceFrames()
	local Frame = PowerBarsDB.PowSet.Frame;
	-- RESOURCE FRAMES
	-- Base Resource Frame
	-- PowerBars.Base.Resource
	PowerBars.Base.Resource = {};
	PowerBars.Base.Resource.Base = CreateFrame("FRAME", nil, UIParent);
	local Base = PowerBars.Base;
	local Resource = PowerBars.Base.Resource;
	Resource.Base.Settings = PowerBarsDB.PowSet.Frame[1];
	CurrFrame[1] = Resource.Base;
	Resource.Base:SetFrameStrata("BACKGROUND");
	Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
	Resource.Base:SetPoint(Frame[1].Point, "UIParent", Frame[1].RelativePoint, Frame[1].X_Ofs, Frame[1].Y_Ofs);
	Resource.Base.Strata = Resource.Base:GetFrameStrata();
	Resource.Base.relativeTo = UIParent;
	Resource.Base:SetScale(Frame[1].Scale);
	if PowerBarsDB.PowSet.Unlocked == 0 then
		Resource.Base.isMovable = true;
		Resource.Base.isRescale = true;
	else
		Resource.Base.isMovable = false;
		Resource.Base.isRescale = false;
	end
	if Class == "WARLOCK" or Class == "PALADIN" then
		for i=1,3 do
			local j;
			if not Resource.Base.Frame then
				j = 1;
				Resource.Base.Frame = {};
			else
				j = #(Resource.Base.Frame)+1;
			end
			Resource.Base.Frame[j] = CreateFrame("FRAME", nil, Resource.Base);
			CurrFrame[i+2] = Resource.Base.Frame[j];
			Resource.Base.Frame[j].Settings = PowerBarsDB.PowSet.Frame[i+2];
			Resource.Base.Frame[j]:SetSize(Frame[i+2].Width, Frame[i+2].Height);
			--MsgLog:AddLine("Current Frame: "..j.." | Scale: "..Frame[i+2].Scale);
			Resource.Base.Frame[j]:SetScale(Frame[i+2].Scale);
			Resource.Base.Frame[j].Strata = Resource.Base.Frame[j]:GetFrameStrata();
			if PowerBarsDB.PowSet.Unlocked == 0 then
				if i == 1 then
					Resource.Base.Frame[j]:SetPoint(Frame[i+2].Point, Resource.Base);
				else
					Resource.Base.Frame[j]:SetPoint(Frame[i+2].Point, Resource.Base.Frame[j-1], Frame[i+2].RelativePoint);
				end
				Resource.Base.Frame[j].isMovable = false;
				Resource.Base.Frame[j].isRescale = false;
			else
				if Frame[i+2].isMoved == 1 then
					Resource.Base.Frame[j]:SetPoint(Frame[i+2].Point, UIParent, Frame[i+2].RelativePoint, Frame[i+2].X_Ofs, Frame[i+2].Y_Ofs);
				else
					if i == 1 then
						Resource.Base.Frame[j]:SetPoint("TOPLEFT", Resource.Base);
					else
						Resource.Base.Frame[j]:SetPoint("LEFT", Resource.Base.Frame[j-1], "RIGHT");
					end
				end
				Resource.Base.Frame[j].isMovable = true;
				Resource.Base.Frame[j].isRescale = true;
			end
			if i == 1 then
				Resource.Base.Frame[j].relativeTo = Resource.Base;
			else
				Resource.Base.Frame[j].relativeTo = Resource.Base.Frame[j-1];
			end
			Resource.Base.Frame[j].Texture = Resource.Base.Frame[j]:CreateTexture(nil, "BACKGROUND");
			Resource.Base.Frame[j].Texture:SetAllPoints(Resource.Base.Frame[j]);
			Resource.Base.Frame[j].Texture:SetAlpha((PowerBarsDB.PowSet.BaseFrameAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
			Resource.Base.Frame[j].Overlay = CreateFrame("BUTTON", nil, Resource.Base.Frame[j]);
			Resource.Base.Frame[j].Overlay:SetAllPoints(Resource.Base.Frame[j]);
			Resource.Base.Frame[j].Overlay:Hide()
			Resource.Base.Frame[j].Overlay.Move = Resource.Base.Frame[j].Overlay:CreateTexture(nil, "OVERLAY");
			Resource.Base.Frame[j].Overlay.Move:SetTexture(255/255,255/255,0/255,0.5);
			Resource.Base.Frame[j].Overlay.Move:SetAllPoints(Resource.Base.Frame[j]);
		end
	end
	if PowerBarsDB.Options.IsMove then
		Resource.Base:EnableMouse(PowerBarsDB.Options.IsMove);
	elseif PowerBarsDB.Options.IsResize then
		Resource.Base:EnableMouse(PowerBarsDB.Options.IsResize);
	else
		Resource.Base:EnableMouse(false);
	end
	if PowerBarsDB.Options.IsMove then
		PowerBars:CheckDrag(Resource.Base, PowerBarsDB.PowSet.Frame[1]);
	end 
	Resource.Base.Texture = Resource.Base:CreateTexture(nil, "BACKGROUND");
	Resource.Base.Texture:SetAlpha(PowerBarsDB.PowSet.BaseFrameAlpha);
	Resource.Base.Texture:SetAllPoints(Resource.Base);
	Resource.Base.Overlay = CreateFrame("BUTTON", nil, Resource.Base);
	Resource.Base.Overlay:SetAllPoints(Resource.Base);
	Resource.Base.Overlay:Hide()
	Resource.Base.Overlay.Move = Resource.Base.Overlay:CreateTexture(nil, "OVERLAY");
	Resource.Base.Overlay.Move:SetAllPoints(Resource.Base);
	Resource.Base.Overlay.Move:SetTexture(255/255,255/255,0/255,0.5);
	if Class == "DRUID" 
	or Class == "PALADIN" 
	or Class == "WARLOCK" then 
		if Class == "WARLOCK" or Class == "PALADIN" then
			for i=1,3 do
				Resource.Base.Frame[i].ResTexture = Resource.Base.Frame[i]:CreateTexture( nil, "ARTWORK");
				Resource.Base.Frame[i].ResTexture:SetAllPoints(Resource.Base.Frame[i]);
				Resource.Base.Frame[i].ResTexture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
			end
		else
			Resource.Base.ResTexture = Resource.Base:CreateTexture( nil, "ARTWORK");
			Resource.Base.ResTexture:SetAllPoints(Resource.Base);
			Resource.Base.ResTexture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * PowerBarsDB.PowSet.ResOOCAlpha));
		end
	end
	if Class == "DEATHKNIGHT" or Class == "PALADIN" or Class == "DRUID" or Class == "WARLOCK" then 
		--Nothing
	else
		PowerBarsDB.PowSet.ResBase = 0;
	end
end

function PowerBars:CreatePowerFrames()
	local Frame = PowerBarsDB.PowSet.Frame;
	-- Power Bars
	local Pow = PowerBars.Pow;
	PowerBars.Base.Power = {};
	local Power = PowerBars.Base.Power;
	Power.StatusBar = CreateFrame("STATUSBAR", nil, UIParent);
	Power.StatusBar.Settings = PowerBarsDB.PowSet.Frame[2];
	CurrFrame[2] = Power.StatusBar;
	Power.StatusBar.Strata = Power.StatusBar:GetFrameStrata();
	Power.StatusBar.relativeTo = UIParent;
	Power.StatusBar.isMovable = true;
	Power.StatusBar.isRescale = true;
	Power.StatusBar:SetSize(Frame[2].Width,Frame[2].Height);
	Power.StatusBar:SetPoint(Frame[2].Point, UIParent, Frame[2].RelativePoint, Frame[2].X_Ofs, Frame[2].Y_Ofs);
	Power.StatusBar:SetOrientation("HORIZONTAL");
	Power.StatusBar:SetStatusBarTexture(IMG.."StatusBar");
	Power.StatusBar:SetAlpha((PowerBarsDB.PowSet.PowAlpha * PowerBarsDB.PowSet.PowOOCAlpha));
	Power.StatusBar:SetScale(Frame[2].Scale);
	Power.StatusBar:SetMovable(PowerBarsDB.Options.IsMove);
	Power.StatusBar:SetResizable(PowerBarsDB.Options.IsResize);
	Power.StatusBar:IsUserPlaced(true);
	if PowerBarsDB.Options.IsMove then
		Power.StatusBar:EnableMouse(PowerBarsDB.Options.IsMove);
	elseif PowerBarsDB.Options.IsResize then
		Power.StatusBar:EnableMouse(PowerBarsDB.Options.IsResize);
	else
		Power.StatusBar:EnableMouse(false);
	end
	if PowerBarsDB.Options.IsMove then
		PowerBars:CheckDrag(Power.StatusBar, PowerBarsDB.PowSet.Frame[2]);
	end
	Power.StatusBar.Overimg = CreateFrame("FRAME", nil, Power.StatusBar);
	local Overimg = Power.StatusBar.Overimg;
	Overimg:SetAllPoints(Power.StatusBar);
	Overimg.Texture = Overimg:CreateTexture(nil, "BACKGROUND");
	Overimg.Texture:SetTexture(IMG.."StatusBarOverlay");
	Overimg.Texture:SetAllPoints(Power.StatusBar);
	Overimg.TextBox = CreateFrame("FRAME", nil, Power.StatusBar);
	Overimg.TextBox:SetPoint("CENTER", Overimg, "CENTER");
	Overimg.TextBox:SetSize(100, 20);
	Overimg.Font = PowerBars:CreateFont(Power.StatusBar, UnitPower("Player", Pow[1].Type).."/"..select(2, Power.StatusBar:GetMinMaxValues()), "DEFAULT", Power.StatusBar:GetHeight()/3, 0, 0, "CENTER", {255,255,255,1}, "DRAWLAYER", "OUTLINE");
	Power.StatusBar.Overlay = CreateFrame("BUTTON", nil, Power.StatusBar);
	Power.StatusBar.Overlay:SetAllPoints(Power.StatusBar);
	Power.StatusBar.Overlay:Hide()
	Power.StatusBar.Overlay.Move = Power.StatusBar.Overlay:CreateTexture(nil, "OVERLAY");
	Power.StatusBar.Overlay.Move:SetAllPoints(Power.StatusBar);
	Power.StatusBar.Overlay.Move:SetTexture(1,1,0,0.5);
end

function PowerBars:SetPowerFrames()
	local Pow = PowerBars.Pow;
	-- POWER FRAMES
 	local Power = PowerBars.Base.Power;
	local Overimg = Power.StatusBar.Overimg;
	
	if PowerBarsDB.PowSet.PowDisable == 0 then
		Power.StatusBar:Show();
	else
		Power.StatusBar:Hide();
	end
	if Class == "DRUID" then
		PowerBars:IfShapeshift("DRUID");
	end
	Overimg.Font:SetText(UnitPower("Player", Pow[1].Type).."/"..UnitPowerMax("Player", Pow[1].Type));
	Overimg.Font:ClearAllPoints();
	Overimg.Font:SetAllPoints(Overimg.TextBox);
	
	if Pow[1].Name == "Mana" then
		Power.StatusBar:SetStatusBarColor(0/255,0/255,255/255,1);
	elseif Pow[1].Name == "Rage" then
		Power.StatusBar:SetStatusBarColor(255/255,0/255,0/255,1);
	elseif Pow[1].Name == "Energy" then
		Power.StatusBar:SetStatusBarColor(255/255,255/255,0/255,1);
	elseif Pow[1].Name == "Runic Power" then
		Power.StatusBar:SetStatusBarColor(1/255,213/255,234/255,1);
	elseif Pow[1].Name == "Focus" then
		Power.StatusBar:SetStatusBarColor(248/255,148/255,30/255,1);
	end
	Power.StatusBar:SetMinMaxValues(0, UnitPowerMax("Player", Pow[1].Type));
	Power.StatusBar:SetValue(UnitPower("Player", Pow[1].Type));
end