local _, Class = UnitClass("Player");
local MsgLog = PowerBars.MsgLog;
local CurrFrame = PowerBars.CurrFrame;

local InCombatLockdown = InCombatLockdown;

function PowerBars.doDruidUnlock(self)
	if self:GetChecked() == 1 then
		PowerBarsDB.Options.DRUID.Unlocked = 1;
		PowerBarsDB.PowSet.Unlocked = 1;
		CurrFrame[1].isMovable = false;
		CurrFrame[1].isRescale = false;
		for i=1,4 do
			if i ~= 2 then
				CurrFrame[i+2].isMovable = true;
				CurrFrame[i+2].isReScale = true;
			end
		end
		if PowerBars.isUnlocked then
			PowerBars:doLock();
			PowerBars:doUnlock()
		end
	else
		PowerBarsDB.Options.DRUID.Unlocked = 0;
		PowerBarsDB.PowSet.Unlocked = 0;
		CurrFrame[1].isMovable = true;
		CurrFrame[1].isRescale = true;
		for i=1,4 do
			if i ~= 2 then
				CurrFrame[i+2].isMovable = false;
				CurrFrame[i+2].isReScale = false;
			end
		end
		if PowerBars.isUnlocked then
			PowerBars:doLock();
			PowerBars:doUnlock()
		end
	end
end

function PowerBars.doRegUnlock(self)
	if self:GetChecked() == 1 then
		PowerBarsDB.Options[Class].Layout = 5;
		PowerBarsDB.PowSet.Unlocked = 1;
		UIDropDownMenu_SetSelectedID(PowerBars.DropMenus[Class].Layout, PowerBarsDB.Options[Class].Layout);
	else
		PowerBarsDB.Options[Class].Layout = 1;
		PowerBarsDB.PowSet.Unlocked = 0;
		local DEFS = PowerBars.Defaults[Class].Settings;
		local Settings = PowerBarsDB.PowSet.Frame;
		for i=1,#CurrFrame do
			if Settings[i].isScaled == 1 then
				Settings[i].Scale = DEFS[i].SCALE;
				CurrFrame[i]:SetScale(Settings[i].Scale);
				Settings[i].isScaled = 0;
			end
			if Settings[i].isMoved == 1 then
				Settings[i].Point = DEFS[i].POINT;
				Settings[i].RelativePoint = DEFS[i].RELATIVEPOINT;
				Settings[i].X_Ofs = DEFS[i].X_OFS;
				Settings[i].Y_Ofs = DEFS[i].Y_OFS;
				CurrFrame[i]:ClearAllPoints();
				CurrFrame[i]:SetPoint(Settings[i].Point, CurrFrame[i].relativeTo, Settings[i].RelativePoint, Settings[i].X_Ofs, Settings[i].Y_Ofs);
				MsgLog:AddLine("Frame: "..i.."| Point: "..Settings[i].Point.." | RelPoint: "..Settings[i].RelativePoint);
				Settings[i].isMoved = 0;
			end
		end
		UIDropDownMenu_SetSelectedID(PowerBars.DropMenus[Class].Layout, PowerBarsDB.Options[Class].Layout);
	end
	if PowerBarsDB.PowSet.Unlocked == 1 then
		for i=1,#CurrFrame do
			if i == 1 then
				CurrFrame[i].isMovable = false;
				CurrFrame[i].isRescale = false;
			elseif i > 2 then
				CurrFrame[i].isMovable = true;
				CurrFrame[i].isRescale = true;
			end
		end
	else
		for i=1,#CurrFrame do
			if i == 1 then
				CurrFrame[i].isMovable = true;
				CurrFrame[i].isRescale = true;
			elseif i > 2 then
				CurrFrame[i].isMovable = false;
				CurrFrame[i].isRescale = false;
			end
		end
	end
	if PowerBars.isUnlocked then
		PowerBars:doLock();
		PowerBars:doUnlock()
	end
end

local DruidUnlockFunc = PowerBars.doDruidUnlock;

function PowerBars:CreatePanel()
	local Resource = PowerBars.Base.Resource;
	-- =========================
	-- Panel Frames and Settings
	-- =========================
	-- Initializing panel frames for later on in the function
	-- PowerBars.panel (parent)
	PowerBars.panel = CreateFrame("FRAME", "PBPanel", UIParent);
	PowerBars.panel.name = "Power Bars";
	InterfaceOptions_AddCategory(PowerBars.panel);
	
	-- PowerBars.Resource (child of PowerBars.panel)
	PowerBars.Resource = CreateFrame("FRAME", "PBResource", PowerBars.panel);
	PowerBars.Resource.name = "Resource Frames";
	PowerBars.Resource.parent = PowerBars.panel.name;
	InterfaceOptions_AddCategory(PowerBars.Resource);
	
	-- PowerBars.Power (child of PowerBars.panel)
	PowerBars.Power = CreateFrame("FRAME", "PBPower", PowerBars.panel);
	PowerBars.Power.name = "Power Frames";
	PowerBars.Power.parent = PowerBars.panel.name;
	InterfaceOptions_AddCategory(PowerBars.Power);

	-- PowerBars.Class (child of PowerBars.panel)
	PowerBars.Class = CreateFrame("FRAME", "PBCustom", PowerBars.panel);
	PowerBars.Class.name = "Class-Specific";
	PowerBars.Class.parent = PowerBars.panel.name;
	InterfaceOptions_AddCategory(PowerBars.Class);
	
	-- PowerBars.Custom (child of PowerBars.panel)
	PowerBars.Custom = CreateFrame("FRAME", "PBCustom", PowerBars.panel);
	PowerBars.Custom.name = "Custom";
	PowerBars.Custom.parent = PowerBars.panel.name;
	InterfaceOptions_AddCategory(PowerBars.Custom);
	
	-- *** GENERAL OPTIONS PANEL ***
	local Panel = {};
	
	-- This frame displays the Power Bars Logo
	Panel.Logo = {};
	local Logo = Panel.Logo;
	Logo.Frame = CreateFrame("FRAME", nil, PowerBars.panel);
	Logo.Frame:SetPoint("TOPLEFT", PowerBars.panel, "TOPLEFT", 20, -10);
	Logo.Frame:SetSize(256,128);
	Logo.Texture = Logo.Frame:CreateTexture("PBLogoTexture", "BACKGROUND");
	Logo.Texture:SetTexture("Interface\\AddOns\\PowerBars\\imgs\\PBLogo");
	Logo.Texture:SetAllPoints(Logo.Frame);
    Logo.Frame:Show();
	
	-- This frame displays introduction text to Power Bars
	Panel.Intro = {};
	local Intro = Panel.Intro;
	Intro.Frame = CreateFrame("FRAME", nil, PowerBars.panel);
	Intro.Frame:SetPoint("TOPLEFT", Logo.Frame, "BOTTOMLEFT", 0, 0);
	Intro.Frame:SetSize(375,100);
	Intro.Text = PowerBars:CreateFont(Intro.Frame, "Power Bars is a Power & Resource display AddOn. Its purpose is to offer customizable displays for all Power and Resource systems in the game, this includes (but isn't necessarily limited to): Mana, Rage, Energy, Runic Power, Focus, Eclipse, Holy Power, and Soul Shards. You may use default frames, custom frames or numbers.\n\nVersion: 0.52 (Beta)", "DEFAULT", 12, 375, 100, "LEFT", {255,255,255,1});
	Intro.Frame:Show();
	
	-- Base Options Frame (GENERAL OPTIONS PANEL)
	Panel.Options = {};
	local Options = Panel.Options;
	Options.Frame = CreateFrame("FRAME", nil, PowerBars.panel);
	Options.Frame:SetPoint("TOPLEFT", Intro.Frame, "BOTTOMLEFT", 0, 0);
	Options.Frame:SetSize(200,20);
	
	-- General Options Title
	Options.Title = PowerBars:CreateFont(Options.Frame, "General Options", "DEFAULT", 20);
	Options.Title:SetPoint("TOPLEFT", Intro.Frame, "BOTTOMLEFT", 0, -10);

	Options.Frame:Show();

	-- Unlock Button
	-- Panel.Options.Unlock
	PowerBars.Buttons.Unlock = PowerBars:CreateButton(PowerBars.panel, "PBUnlock", "Unlock");
	Options.Unlock = PowerBars.Buttons.Unlock;
	Options.Unlock:ClearAllPoints();
	Options.Unlock:SetPoint("TOPLEFT", Options.Title, "BOTTOMLEFT", 0, -10);
	Options.Unlock:SetScript("OnClick", 
	function()
		PowerBars:doUnlock();
		PowerBars:ShowUnlock();
	end);
	
	Options.Check = PowerBars.Check;
	Options.Check.General = {};
	
	Options.Check.General.ShowError = PowerBars:CreateCheck(Options.Frame, "PBCheck_ShowError", "Show Error Details");
	local ShowError = Options.Check.General.ShowError;
	ShowError.tooltip = "Brings up the Power Bars Message Console whenever Power Bars throws an internal error. Information from this console can be used to give more detailed feedback when reporting errors to the author.";
	ShowError:ClearAllPoints();
	ShowError:SetPoint("TOPLEFT", Options.Unlock, "BOTTOMLEFT", 0, -5);
	
	ShowError:SetScript("OnClick",
		function(self)
			if self:GetChecked() == 1 then
				PowerBarsDB.Options.ErrorConsole = 1;
			else
				PowerBarsDB.Options.ErrorConsole = 0;
			end
		end);
	
	if PowerBarsDB.Options.ErrorConsole == 1 then
		ShowError:SetChecked(true);
	else
		ShowError:SetChecked(false);
	end
	
	-- *** RESOURCE PANEL ***
	local ResourceOptions = PowerBars.Resource;
	ResourceOptions.Titles = {};
	
	-- Setup Main Title "Resource Frame Options"
	ResourceOptions.Titles.Main = PowerBars:CreateFont(ResourceOptions, "Resource Frame Options", "DEFAULT", 20);
	ResourceOptions.Titles.Main:ClearAllPoints();
	ResourceOptions.Titles.Main:SetPoint("TOPLEFT", ResourceOptions, "TOPLEFT", 10, -10);
	
	-- Setup Sub-Title "Toggles"
	ResourceOptions.Titles.Toggles = PowerBars:CreateFont(ResourceOptions, "Toggles", "DEFAULT", 16);
	ResourceOptions.Titles.Toggles:ClearAllPoints();
	ResourceOptions.Titles.Toggles:SetPoint("TOPLEFT", ResourceOptions.Titles.Main, "BOTTOMLEFT", 0, -10);
	
	-- || Check Button Initialization ||
	-- EnableResource: Toggles whether or not you see Base.Resource.Base, which contains all the children components of a Resouce display;
	PowerBars.Check.DisableResource = PowerBars:CreateCheck(ResourceOptions, "PBCheck_DisableResource", "Disable resource frame", "TOP", ResourceOptions.Titles.Toggles, "BOTTOMLEFT", 15, -10);
	local DisableResource = PowerBars.Check.DisableResource;
	DisableResource.tooltip = "Toggles the Power Bars resource frame. (Holy Power, Soul Shards, Eclipse Bar, or Rune Bar)";
	
	PowerBars.Check.DisableBlizzResource = PowerBars:CreateCheck(ResourceOptions, "PBCheck_DisableBlizzResource", "Disable default resource frame", "TOP", DisableResource, "BOTTOM");
	local DisableBlizzResource = PowerBars.Check.DisableBlizzResource;
	DisableBlizzResource.tooltip = "Toggles Blizzard's default resource frame. (Holy Power, Soul Shards, Eclipse Bar, or Rune Bar)";
	
	-- BaseResTexture: Toggles the Base.Resource.Texture for Paladins and Warlocks
	PowerBars.Check.BaseResTexture = PowerBars:CreateCheck(ResourceOptions, "PBCheck_BaseResTexture", "Toggle '0 Power Frame'", "TOP", DisableBlizzResource, "BOTTOM");
	local BaseResTexture = PowerBars.Check.BaseResTexture;
	BaseResTexture.tooltip = "Allows you to toggle whether or not you see anything when at 0 of your resource.";
	
	-- Setup Sub-Title "Display"
	ResourceOptions.Titles.Display = PowerBars:CreateFont(ResourceOptions, "Display Options", "DEFAULT", 16);
	ResourceOptions.Titles.Display:ClearAllPoints();
	ResourceOptions.Titles.Display:SetPoint("TOPLEFT", BaseResTexture, "BOTTOMLEFT", 0, -20);
	
	-- Opacity Slider for '0 power' image
	PowerBars.Sliders.BaseOpacity = PowerBars:CreateSlider(ResourceOptions, "PBSliders_BaseOpacity", "'0 power' Opacity: "..(math.floor(PowerBarsDB.PowSet.BaseFrameAlpha*100)), 0, 1, "1");
	local BaseOpacity = {};
	BaseOpacity.Slider = PowerBars.Sliders.BaseOpacity;
	BaseOpacity.Slider:SetValueStep(0.01);
	BaseOpacity.Slider:SetPoint("TOPLEFT", ResourceOptions.Titles.Display, "BOTTOMLEFT", 0, -15);
	BaseOpacity.Slider:SetValue(PowerBarsDB.PowSet.BaseFrameAlpha);
	BaseOpacity.Slider.isDragging = 0;
	
	-- Resource Texture Opacity Slider
	PowerBars.Sliders.ResOpacity = PowerBars:CreateSlider(ResourceOptions, "PBSliders_ResOpacity", "Resource Frame Opacity: "..(math.floor(PowerBarsDB.PowSet.ResAlpha*100)), 0, 1, "1");
	local ResOpacity = PowerBars.Sliders.ResOpacity;
	ResOpacity:SetValueStep(0.01);
	ResOpacity:SetPoint("TOPLEFT", BaseOpacity.Slider, "BOTTOMLEFT", 0, -15);
	ResOpacity:SetValue(PowerBarsDB.PowSet.ResAlpha);
	ResOpacity.isDragging = 0;
	
	-- Resource Texture OOC Opacity Slider
	PowerBars.Sliders.ResOOCOpacity = PowerBars:CreateSlider(ResourceOptions, "PBSliders_ResOOCOpacity", "Resource Frame OOC Opacity: "..(math.floor(PowerBarsDB.PowSet.ResOOCAlpha*100)), 0, 1, "1");
	local ResOOCOpacity = PowerBars.Sliders.ResOOCOpacity;
	ResOOCOpacity:SetValueStep(0.01);
	ResOOCOpacity:SetPoint("TOPLEFT", ResOpacity, "BOTTOMLEFT", 0, -15);
	ResOOCOpacity:SetValue(PowerBarsDB.PowSet.ResOOCAlpha);
	ResOOCOpacity.isDragging = 0;
	
	-- || Check Button Scripts ||
	DisableResource:SetScript("OnClick",
		function()
			local IsCheck = DisableResource:GetChecked();
			if IsCheck == 1 then
				PowerBarsDB.PowSet.ResBase = 0;
				Resource.Base:Hide();
			else
				PowerBarsDB.PowSet.ResBase = 1;
				Resource.Base:Show();
			end
		end);
		
	DisableBlizzResource:SetScript("OnClick",
		function()
			local IsCheck = DisableBlizzResource:GetChecked();
			if IsCheck == 1 then
				PowerBarsDB.PowSet.BlizzResDisable = 1;
				PowerBars.BlizzResFrame:Hide();
			else
				PowerBarsDB.PowSet.BlizzResDisable = 0;
				PowerBars.BlizzResFrame:Show();
			end
		end);
		
	BaseResTexture:SetScript("OnClick",
		function()
			local IsCheck = BaseResTexture:GetChecked();
			if IsCheck == 1 then
				PowerBarsDB.PowSet.ResTexture = 1;
				local img = IMG .. 	PowerBarsDB.PowSet.PowIMG;
				for i=1,3 do
					local BaseIMG = img .. i-1;
					Base.Resource.Base.Frame[i].Texture:SetTexture(BaseIMG);
				end
			else
				PowerBarsDB.PowSet.ResTexture = 0;
				for i=1,3 do
					Base.Resource.Base.Frame[i].Texture:SetTexture("");
				end
			end
		end);
	
	if Class == "WARLOCK" or Class == "PALADIN" or Class == "DRUID" or Class == "DEATHKNIGHT" then 
		DisableResource:Enable();
		DisableBlizzResource:Enable();
		if PowerBarsDB.PowSet.ResBase == 0 then
			DisableResource:SetChecked(true);
		else
			DisableResource:SetChecked(false);
		end
		if PowerBarsDB.PowSet.BlizzResDisable == 1 then
			DisableBlizzResource:SetChecked(true);
		else
			DisableBlizzResource:SetChecked(false);
		end
	else
		DisableResource:Disable();
		DisableBlizzResource:Disable();
	end
		
	if Class == "WARLOCK" or Class == "PALADIN" then 
		BaseResTexture:Enable();
		if PowerBarsDB.PowSet.ResTexture == 1 then
			BaseResTexture:SetChecked(true);
		else
			BaseResTexture:SetChecked(false);
		end
	else
		BaseResTexture:Disable();
	end
	
	-- || Slider Scripts ||
	BaseOpacity.Slider:SetScript("OnMouseDown",
		function(self)
			self.isDragging = 1;
		end);
		
	BaseOpacity.Slider:SetScript("OnUpdate",
		function(self)
			if self.isDragging == 1 then
				_G[self:GetName() .. 'Text']:SetText("'0 power' Opacity: "..(math.floor((self:GetValue()*100)+0.5)));
				if Class == "PALADIN" or Class == "WARLOCK" then
					for i=1,3 do
						Resource.Base.Frame[i].Texture:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));	
					end
				else
					Resource.Texture:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
				end
			end
		end); 
		
	BaseOpacity.Slider:SetScript("OnMouseUp",
		function(self)
			self.isDragging = 0;
			_G[self:GetName() .. 'Text']:SetText("'0 power' Opacity: "..(math.floor(self:GetValue()*100)));
			PowerBarsDB.PowSet.BaseFrameAlpha = self:GetValue();
			local inCombat = InCombatLockdown()
			if inCombat then
				PowerBars:AdjustOOCAlpha(1);
			else
				PowerBars:AdjustOOCAlpha(0);
			end
		end);
	
	ResOpacity:SetScript("OnMouseDown",
		function(self)
			self.isDragging = 1;
		end);
		
	ResOpacity:SetScript("OnUpdate",
		function(self)
			if self.isDragging == 1 then
				_G[self:GetName() .. 'Text']:SetText("Resource Frame Opacity: "..	(math.floor((self:GetValue()*100)+0.5)));
				if Class == "PALADIN" or Class == "WARLOCK" then
					for i=1,3 do
						Resource.Base.Frame[i].ResTexture:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));	
					end
				elseif Class == "DEATHKNIGHT" then
					local Runes = PowerBars.Runes;
					for i=1,6 do
						local Rune = Runes[i];
						local Texture = Rune.Frame.Texture
						--Texture:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
						Rune.Frame:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
					end
				elseif Class == "DRUID" and GetPrimaryTalentTree() == 1 then
					Resource.ResTexture:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
					PowerBars.Eclipse.Arrow.Texture:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
					PowerBars.Eclipse.Text.Font:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
					PowerBars.Eclipse.vText.Font:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
				end
			end
		end); 
		
	ResOpacity:SetScript("OnMouseUp",
		function(self)
			self.isDragging = 0;
			_G[self:GetName() .. 'Text']:SetText("Resource Frame Opacity: "..(math.floor(self:GetValue()*100)));
			PowerBarsDB.PowSet.ResAlpha = self:GetValue();
			local inCombat = InCombatLockdown()
			if inCombat then
				PowerBars:AdjustOOCAlpha(1);
			else
				PowerBars:AdjustOOCAlpha(0);
			end
		end);
		
	ResOOCOpacity:SetScript("OnMouseDown",
		function(self)
			self.isDragging = 1;
		end);
	
	ResOOCOpacity:SetScript("OnUpdate",
		function(self)
			if self.isDragging == 1 then
				_G[self:GetName() .. 'Text']:SetText("Resource Frame OOC Opacity: "..(math.floor((self:GetValue()*100)+0.5)));
				if Class == "PALADIN" or Class == "WARLOCK" then
					for i=1,3 do
						Resource.Base.Frame[i].ResTexture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * self:GetValue()));
						Resource.Base.Frame[i].Texture:SetAlpha((PowerBarsDB.PowSet.BaseFrameAlpha * self:GetValue()));
					end
				elseif Class == "DEATHKNIGHT" then
					local Runes = PowerBars.Runes;
					for i=1,6 do
						local Rune = Runes[i];
						local Texture = Rune.Frame.Texture
						--Texture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * self:GetValue()));
						Rune.Frame:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.ResOOCAlpha));
					end
				elseif Class == "DRUID" and GetPrimaryTalentTree() == 1 then
					Resource.ResTexture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * self:GetValue()));
					PowerBars.Eclipse.Arrow.Texture:SetAlpha((PowerBarsDB.PowSet.ResAlpha * self:GetValue()));
					PowerBars.Eclipse.Text.Font:SetAlpha((PowerBarsDB.PowSet.ResAlpha * self:GetValue()));
					PowerBars.Eclipse.vText.Font:SetAlpha((PowerBarsDB.PowSet.ResAlpha * self:GetValue()));
				end
			end
		end); 	
	
	ResOOCOpacity:SetScript("OnMouseUp",
		function(self)
			self.isDragging = 0;
			_G[self:GetName() .. 'Text']:SetText("Resource Frame OOC Opacity: "..(math.floor(self:GetValue()*100)));
			PowerBarsDB.PowSet.ResOOCAlpha = self:GetValue();
			local inCombat = InCombatLockdown()
			if inCombat then
				PowerBars:AdjustOOCAlpha(1);
			else
				PowerBars:AdjustOOCAlpha(0);
			end
		end);
	
	if Class == "WARLOCK" or Class == "PALADIN" then
		BaseOpacity.Slider:Enable();
	else
		BaseOpacity.Slider:Disable();
	end
	
	if Class == "DRUID" or Class == "PALADIN" or Class == "DEATHKNIGHT" or Class == "WARLOCK" then
		ResOpacity:Enable();
		ResOOCOpacity:Enable();
	else
		ResOpacity:Disable();
		ResOOCOpacity:Disable();
	end
	
	-- || Edit Box Scripts ||

		
	-- *** POWER PANEL ***
	local PowerOptions = PowerBars.Power
	PowerOptions.Titles = {};
	
	-- Creating the main title
	PowerOptions.Titles.Main = PowerBars:CreateFont(PowerOptions, "Power Frame Options", "DEFAULT", 20);
	PowerOptions.Titles.Main:ClearAllPoints();
	PowerOptions.Titles.Main:SetPoint("TOPLEFT", PowerOptions, "TOPLEFT", 10, -10);
	
	-- Setup Sub-Title "Toggles"
	PowerOptions.Titles.Toggles = PowerBars:CreateFont(PowerOptions, "Toggles", "DEFAULT", 16);
	PowerOptions.Titles.Toggles:ClearAllPoints();
	PowerOptions.Titles.Toggles:SetPoint("TOPLEFT", PowerOptions.Titles.Main, "BOTTOMLEFT", 0, -10);
	
	-- || Check Button Initialization ||
	PowerBars.Check.DisablePow = PowerBars:CreateCheck(PowerOptions, "PBCheck_DisablePow", "Disable power bars", "TOP", PowerOptions.Titles.Toggles, "BOTTOMLEFT", 15, -10);
	local DisablePow = PowerBars.Check.DisablePow;
	DisablePow.tooltip = "Toggles where or not power bars display. Power bars include: Mana, Rage, Runic Power, Focus, and Energy."
	
	-- Power Texture Opacity Slider
	PowerBars.Sliders.PowOpacity = PowerBars:CreateSlider(PowerOptions, "PBSliders_PowOpacity", "Power Frame Opacity: "..(math.floor(PowerBarsDB.PowSet.PowAlpha*100)), 0, 1, "1");
	local PowOpacity = PowerBars.Sliders.PowOpacity;
	PowOpacity:SetValueStep(0.01);
	PowOpacity:SetPoint("TOPLEFT", DisablePow, "BOTTOMLEFT", 0, -15);
	PowOpacity:SetValue(PowerBarsDB.PowSet.PowAlpha);
	PowOpacity.isDragging = 0;
	
	-- Power Texture OOC Opacity Slider
	PowerBars.Sliders.PowOOCOpacity = PowerBars:CreateSlider(PowerOptions, "PBSliders_PowOOCOpacity", "Power Frame OOC Opacity: "..(math.floor(PowerBarsDB.PowSet.PowOOCAlpha*100)), 0, 1, "1");
	local PowOOCOpacity = PowerBars.Sliders.PowOOCOpacity;
	PowOOCOpacity:SetValueStep(0.01);
	PowOOCOpacity:SetPoint("TOPLEFT", PowOpacity, "BOTTOMLEFT", 0, -15);
	PowOOCOpacity:SetValue(PowerBarsDB.PowSet.PowOOCAlpha);
	PowOOCOpacity.isDragging = 0;
	
	-- || Check Button Scripts ||
	DisablePow:SetScript("OnClick",
		function(self)
			local IsCheck = self:GetChecked();
			if IsCheck == 1 then
				PowerBarsDB.PowSet.PowDisable = 1;
				PowOpacity:Disable()
				PowOOCOpacity:Disable()
				if PowerBars.Base.Power.StatusBar then
					PowerBars.Base.Power.StatusBar:Hide();
				end
			else
				PowerBarsDB.PowSet.PowDisable = 0;
				PowOpacity:Enable()
				PowOOCOpacity:Enable()
				if PowerBars.Base.Power.StatusBar then
					PowerBars.Base.Power.StatusBar:Show();
				end
			end
		end);
	
	-- || Slider Scripts ||
	PowOpacity:SetScript("OnMouseDown",
		function(self)
			self.isDragging = 1;
		end);
		
	PowOpacity:SetScript("OnUpdate",
		function(self)
			if self.isDragging == 1 then
				_G[self:GetName() .. 'Text']:SetText("Power Frame Opacity: "..	(math.floor((self:GetValue()*100)+0.5)));
				PowerBars.Base.Power.StatusBar:SetAlpha((self:GetValue() * PowerBarsDB.PowSet.PowOOCAlpha));
			end
		end); 
		
	PowOpacity:SetScript("OnMouseUp",
		function(self)
			self.isDragging = 0;
			_G[self:GetName() .. 'Text']:SetText("Power Frame Opacity: "..(math.floor(self:GetValue()*100)));
			PowerBarsDB.PowSet.PowAlpha = self:GetValue();
			local inCombat = InCombatLockdown()
			if inCombat then
				PowerBars:AdjustOOCAlpha(1);
			else
				PowerBars:AdjustOOCAlpha(0);
			end
		end);
		
	PowOOCOpacity:SetScript("OnMouseDown",
		function(self)
			self.isDragging = 1;
		end);
	
	PowOOCOpacity:SetScript("OnUpdate",
		function(self)
			if self.isDragging == 1 then
				_G[self:GetName() .. 'Text']:SetText("Power Frame OOC Opacity: "..(math.floor((self:GetValue()*100)+0.5)));
				PowerBars.Base.Power.StatusBar:SetAlpha((PowerBarsDB.PowSet.PowAlpha * self:GetValue()));
			end
		end); 	
	
	PowOOCOpacity:SetScript("OnMouseUp",
		function(self)
			self.isDragging = 0;
			_G[self:GetName() .. 'Text']:SetText("Power Frame OOC Opacity: "..(math.floor(self:GetValue()*100)));
			PowerBarsDB.PowSet.PowOOCAlpha = self:GetValue();
			local inCombat = InCombatLockdown()
			if inCombat then
				PowerBars:AdjustOOCAlpha(1);
			else
				PowerBars:AdjustOOCAlpha(0);
			end
		end);
	
	if PowerBarsDB.PowSet.PowDisable == 1 then
		DisablePow:SetChecked(true);
		PowOpacity:Disable()
		PowOOCOpacity:Disable()
	else
		DisablePow:SetChecked(false);
		PowOpacity:Enable()
		PowOOCOpacity:Enable()
	end
	
	-- *** CLASS PANEL ***
	-- | Death Knight |
	if Class == "DEATHKNIGHT" then
		PowerBars.Class.DEATHKNIGHT = CreateFrame("FRAME", nil, PowerBars.Class);
		local DKOptions = PowerBars.Class.DEATHKNIGHT;
		DKOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		DKOptions:SetSize(375, 100);
		DKOptions.Title = PowerBars:CreateFont(DKOptions, "Death Knights", "DEFAULT", 18, 0, 0, "LEFT", {196,30,59,1});
		DKOptions.Title:SetPoint("TOP", DKOptions, "LEFT");
		DKOptions.Check = PowerBars.Check;
		DKOptions.Check.Timers = PowerBars:CreateCheck(DKOptions, "DKTimers", "Toggle Rune Timers", "TOPLEFT", DKOptions.Title, "BOTTOMLEFT");
		local DKTimers = DKOptions.Check.Timers;
		DKTimers.tooltip = "Allows you to toggle numeric timers. Numeric timers, if toggled on, will be overlayed on your runes. You may choose to overwrite the default numeric timers with OmniCC ones.";
		DKOptions.Check.OmniCC = PowerBars:CreateCheck(DKOptions, "DKOmniCC", "Toggle OmniCC", "TOP", DKTimers, "BOTTOMRIGHT");
		local OmniCC = DKOptions.Check.OmniCC;
		OmniCC.tooltip = "When checked, the Addon will substitute OmniCC timers instead of the default ones for Power Bars. Unchecked means you will use Power Bar's timers."
		DKOptions.Check.ShowCD = PowerBars:CreateCheck(DKOptions, "ShowCD", "Toggle Cooldown Frame");
		local DKShowCD = DKOptions.Check.ShowCD;
		DKShowCD.tooltip = "Toggles whether or not you see a square 'Blizzard' cooldown frame when runes go on cooldown."
		DKShowCD:ClearAllPoints();
		DKShowCD:SetPoint("TOPRIGHT", OmniCC, "BOTTOMLEFT", 12, 0);
		OmniCC:Disable();
	
		if not PowerBars.DropMenus.DKLayout then
			PowerBars.DropMenus.DEATHKNIGHT.Layout = CreateFrame("FRAME", "PB_DKLayout", DKOptions, "UIDropDownMenuTemplate");
		end
		local DKLayout = PowerBars.DropMenus.DEATHKNIGHT.Layout;
	
		DKLayout:ClearAllPoints();
		DKLayout:SetPoint("LEFT", DKTimers, "RIGHT", 125, 0);
		DKLayout:Show();
		
		function DKLayout.SelectLayout(self, arg1)
			local Frame = PowerBarsDB.PowSet.Frame;
			local Settings = PowerBars.Defaults.DEATHKNIGHT.Settings[1];
			UIDropDownMenu_SetSelectedID(DKLayout, self:GetID());
			if arg1 == "3-2" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.DEATHKNIGHT.Layout = 1;
				Frame[1].Width = Settings.WIDTH;
				Frame[1].Height = Settings.HEIGHT;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,6 do
					local RuneF = PowerBarsDB.PowSet.Frame[i+2];
					RuneF.Scale = 1;
					if i == 1 then
						RuneF.Point = "TOPLEFT";
					elseif i % 2 == 0 then
						RuneF.Point = "LEFT";
						RuneF.RelativePoint = "RIGHT";
					else
						RuneF.Point = "TOP";
						RuneF.RelativePoint = "BOTTOM";
					end
				end
				PowerBars:doRuneFrames();
			elseif arg1 == "2-3" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.DEATHKNIGHT.Layout = 2;
				Frame[1].Width = Settings.HEIGHT;
				Frame[1].Height = Settings.WIDTH;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,6 do
					local RuneF = PowerBarsDB.PowSet.Frame[i+2];
					RuneF.Scale = 1;
					if i == 1 then
						RuneF.Point = "TOPLEFT";
					elseif i % 2 == 0 then
						RuneF.Point = "TOP";
						RuneF.RelativePoint = "BOTTOM";
					else
						RuneF.Point = "LEFT";
						RuneF.RelativePoint = "RIGHT";
					end
				end
				PowerBars:doRuneFrames();
			elseif arg1 == "1-6" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.DEATHKNIGHT.Layout = 3;
				Frame[1].Width = 240;
				Frame[1].Height = 40;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,6 do
					local RuneF = PowerBarsDB.PowSet.Frame[i+2];
					RuneF.Scale = 1;
					if i == 1 then
						RuneF.Point = "TOPLEFT";
					elseif i % 2 == 0 then
						RuneF.Point = "LEFT";
						RuneF.RelativePoint = "RIGHT";
					else
						RuneF.Point = "LEFT";
						RuneF.RelativePoint = "RIGHT";
					end
				end
				PowerBars:doRuneFrames();
			elseif arg1 == "6-1" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.DEATHKNIGHT.Layout = 4;
				Frame[1].Width = 40;
				Frame[1].Height = 240;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,6 do
					local RuneF = PowerBarsDB.PowSet.Frame[i+2];
					RuneF.Scale = 1;
					if i == 1 then
						RuneF.Point = "TOPLEFT";
					elseif i % 2 == 0 then
						RuneF.Point = "TOP";
						RuneF.RelativePoint = "BOTTOM";
					else
						RuneF.Point = "TOP";
						RuneF.RelativePoint = "BOTTOM";
					end 
				end
				PowerBars:doRuneFrames();
			elseif arg1 == "unlock" then
				PowerBarsDB.Options.DEATHKNIGHT.Layout = 5;
				PowerBarsDB.PowSet.Unlocked = 1;
				local Runes = PowerBars.Runes;
				for i=1,6 do
					local Rune = PowerBars.Runes[i].Frame;
					Rune:ClearAllPoints();
				end 
			end	
			if PowerBarsDB.PowSet.Unlocked == 1 then
				for i=1,#CurrFrame do
					if i == 1 then
						CurrFrame[i].isMovable = false;
						CurrFrame[i].isRescale = false;
					elseif i > 2 then
						CurrFrame[i].isMovable = true;
						CurrFrame[i].isRescale = true;
					end
				end
			else
				for i=1,#CurrFrame do
					if i == 1 then
						CurrFrame[i].isMovable = true;
						CurrFrame[i].isRescale = true;
					elseif i > 2 then
						CurrFrame[i].isMovable = false;
						CurrFrame[i].isRescale = false;
					end
				end
			end
			if PowerBars.isUnlocked then
				PowerBars:doLock();
				PowerBars:doUnlock()
			end
		end
		
		DKLayout.Items = {
			{ text = "3 Rows - 2 Columns", arg1 = "3-2" },
			{ text = "2 Rows - 3 Columns", arg1 = "2-3" },
			{ text = "1 Row - 6 Columns", arg1 = "1-6" },
			{ text = "6 Rows - 1 Column", arg1 = "6-1" },
			{ text = "Unlocked", arg1 = "unlock" },
		};
		
		function DKLayout.Initialize(self, level)
			local items = DKLayout.Items;
			local info = UIDropDownMenu_CreateInfo();
			for i=1,#(DKLayout.Items) do
				info = UIDropDownMenu_CreateInfo();
				info.text = items[i].text;
				info.arg1 = items[i].arg1;
				info.func = DKLayout.SelectLayout;
				UIDropDownMenu_AddButton(info, level);
			end
		end
		
		UIDropDownMenu_Initialize(DKLayout, DKLayout.Initialize)
		UIDropDownMenu_SetWidth(DKLayout, 150);
		UIDropDownMenu_SetButtonWidth(DKLayout, 200);
		if not PowerBarsDB.Options.DEATHKNIGHT then
			UIDropDownMenu_SetSelectedID(DKLayout, 1);
		else
			UIDropDownMenu_SetSelectedID(DKLayout, PowerBarsDB.Options.DEATHKNIGHT.Layout);
		end
		UIDropDownMenu_JustifyText(DKLayout, "LEFT");
	
		OmniCC:SetScript("OnClick",
			function()
				local IsCheck = OmniCC:GetChecked();
				if IsCheck == 1 then
					PowerBarsDB.Options.DEATHKNIGHT.OmniCC = 1;
					for i=1,6 do
						PowerBars.Runes[i].Frame.Cooldown.noCooldownCount = false;
					end
				else
					PowerBarsDB.Options.DEATHKNIGHT.OmniCC = 0;
					for i=1,6 do
						PowerBars.Runes[i].Frame.Cooldown.noCooldownCount = true;
					end	
				end
			end);
		
		DKTimers:SetScript("OnClick",
			function()
				local IsCheck = DKTimers:GetChecked();
				if IsCheck == 1 then
					PowerBarsDB.Options.DEATHKNIGHT.Timers = 1;
					OmniCC:Enable();
				else
					PowerBarsDB.Options.DEATHKNIGHT.Timers = 0;
					OmniCC:Disable();
					for i=1,6 do
						if PowerBars.Runes then
							PowerBars.Runes[i].Frame.Cooldown.Frame:Hide();
						end
					end
				end
			end);
		
		DKShowCD:SetScript("OnClick",
			function()
				local IsCheck = DKShowCD:GetChecked();
				if IsCheck == 1 then
					PowerBarsDB.Options.DEATHKNIGHT.ShowCD = 1;
					local Runes = PowerBars.Runes;
					for i=1,6 do
						local Rune = Runes[i];
						local Cooldown = Rune.Frame.Cooldown;
						Cooldown:SetAlpha(1);
					end
				else
					PowerBarsDB.Options.DEATHKNIGHT.ShowCD = 0;
					local Runes = PowerBars.Runes;
					for i=1,6 do
						local Rune = Runes[i];
						local Cooldown = Rune.Frame.Cooldown;
						Cooldown:SetAlpha(0);
					end
				end
			end);
	
		if Class ~= "DEATHKNIGHT" then	
			DKTimers:Disable();
			OmniCC:Disable();
			DKShowCD:Disable();
			UIDropDownMenu_DisableDropDown(DKLayout);
			DKOptions:Hide();
		else
			DKOptions:Show();
			DKTimers:Enable();
			DKShowCD:Enable();
			UIDropDownMenu_EnableDropDown(DKLayout);
			if IsAddOnLoaded("OmniCC") or IsAddOnLoaded("TullaCooldownCount") then
				OmniCC:Enable();
			else
				OmniCC:Disable();
			end
			if PowerBarsDB.Options.DEATHKNIGHT.Timers == 1 then
				DKTimers:SetChecked(true);
			else
				DKTimers:SetChecked(false);
			end
			if PowerBarsDB.Options.DEATHKNIGHT.OmniCC == 1 then
				OmniCC:SetChecked(true);
			else
				OmniCC:SetChecked(false);
			end
			if PowerBarsDB.Options.DEATHKNIGHT.ShowCD == 1 then
				DKShowCD:SetChecked(true);
			else
				DKShowCD:SetChecked(false);
			end
		end
	end
	
	-- | Paladin |
	if Class == "PALADIN" then
		PowerBars.Class.PALADIN = CreateFrame("FRAME", nil, PowerBars.Class);
		local PallyOptions = PowerBars.Class.PALADIN;
		PallyOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		PallyOptions:SetSize(375, 100);
		PallyOptions.Title = PowerBars:CreateFont(PallyOptions, "Paladins", "DEFAULT", 	18, 0, 0, "LEFT", {245,140,186,1});
		PallyOptions.Title:SetPoint("TOP", PallyOptions, "LEFT");
		PallyOptions.Check = PowerBars.Check;
	
		if not PowerBars.DropMenus.PallyLayout then
			PowerBars.DropMenus.PALADIN.Layout = CreateFrame("FRAME", "PB_PallyLayout", 	PallyOptions, "UIDropDownMenuTemplate");
		end
	
		local PallyLayout = PowerBars.DropMenus.PALADIN.Layout;
		PallyLayout:ClearAllPoints();
		PallyLayout:SetPoint("TOPLEFT", PallyOptions.Title, "BOTTOMLEFT", 0, -10);
		PallyLayout:Show();
	
		PallyOptions.Check.NumHPW = PowerBars:CreateCheck(PallyOptions, "PBChecks_NumHPW", "Toggle HPW filter");
		local NumHPW = PallyOptions.Check.NumHPW;
		NumHPW:ClearAllPoints();
		NumHPW:SetPoint("LEFT", PallyLayout, "RIGHT", 10, 0);
		PallyOptions.Edits = PowerBars.Edits;
		PallyOptions.Sliders = PowerBars.Sliders;
		PallyOptions.Sliders.NumHPW = PowerBars:CreateSlider(PallyOptions, "PBSliders_NumHPW", "Show HPW at: ", 1, 3, "1", "3", 100);
		local SliderNumHPW = PallyOptions.Sliders.NumHPW;
		PallyOptions.SubTitles = {};
		PallyOptions.SubTitles.SliderTwo = PowerBars:CreateFont(PallyOptions, "2", "DEFAULT", 10, 0, 0, "LEFT", {255,255,255,1});
		PallyOptions.SubTitles.SliderTwo:ClearAllPoints();
		SliderNumHPW:ClearAllPoints();
		SliderNumHPW:SetPoint("TOPLEFT", NumHPW, "BOTTOMRIGHT", 0, -10);
		PallyOptions.SubTitles.SliderTwo:SetPoint("TOP", SliderNumHPW, "BOTTOM", 0, 3);
		PallyOptions.SubTitles.SliderTwo:SetShadowColor(0,0,0,1);
		PallyOptions.SubTitles.SliderTwo:SetShadowOffset(1,-1);
		SliderNumHPW:SetValueStep(1);
		if Class == "PALADIN" then
			_G[SliderNumHPW:GetName() .. 'Text']:SetText("Show HPW at: "..PowerBarsDB.	Options.PALADIN.NumHPW);
		end
		
		SliderNumHPW:SetScript("OnUpdate",
			function(self)
				if Class == "PALADIN" then
					_G[SliderNumHPW:GetName() .. 'Text']:SetText("Show HPW at: "..self:GetValue());
				end
			end);	
	
		SliderNumHPW:SetScript("OnMouseUp",
			function(self)
				if Class == "PALADIN" then
					_G[SliderNumHPW:GetName() .. 'Text']:SetText("Show HPW at: "..self:	GetValue());
					PowerBarsDB.Options.PALADIN.NumHPW = self:GetValue();
					ChangePowPaladin();
				end
			end);
	
		NumHPW:SetScript("OnClick",
			function(self)
				local IsCheck = self:GetChecked();
				if IsCheck == 1 then
					PowerBarsDB.Options.PALADIN.NumHPW = 1;
					ChangePowPaladin();
				else
					PowerBarsDB.Options.PALADIN.NumHPW = 0;
					ChangePowPaladin();
				end
			end);
	
		if Class == "PALADIN" then
			NumHPW:Enable();
			SliderNumHPW:Enable();
			SliderNumHPW:SetValue(PowerBarsDB.Options.PALADIN.NumHPW);
			if PowerBarsDB.Options.PALADIN.NumHPW > 0 then
				NumHPW:SetChecked(true);
			else
				NumHPW:SetChecked(false);
			end
		else
			NumHPW:Disable();
			SliderNumHPW:Disable();
		end
		
		function PallyLayout.SelectLayout(self, arg1)
			local Frame = PowerBarsDB.PowSet.Frame;
			local Settings = PowerBars.Defaults.PALADIN.Settings[1];
			UIDropDownMenu_SetSelectedID(PallyLayout, self:GetID());
			if arg1 == "Horiz1" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.PALADIN.Layout = 1;
				Frame[1].Width = Settings.WIDTH;
				Frame[1].Height = Settings.HEIGHT;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "TOPLEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "LEFT";
						Settings.RelativePoint = "RIGHT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], 	Settings.RelativePoint);
					end
				end
			elseif arg1 == "Horiz2" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.PALADIN.Layout = 2;
				Frame[1].Width = Settings.WIDTH;
				Frame[1].Height = Settings.HEIGHT;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "TOPRIGHT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "RIGHT";
						Settings.RelativePoint = "LEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], Settings.RelativePoint);
					end
				end
			elseif arg1 == "Vertic1" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.PALADIN.Layout = 3;
				Frame[1].Width = 128;
				Frame[1].Height = 192;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "TOPLEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "TOP";
						Settings.RelativePoint = "BOTTOM";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], 	Settings.RelativePoint);
					end
				end
			elseif arg1 == "Vertic2" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.PALADIN.Layout = 4;
				Frame[1].Width = 128;
				Frame[1].Height = 192;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "BOTTOMLEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "BOTTOM";
						Settings.RelativePoint = "TOP";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], Settings.RelativePoint);
					end
				end
			elseif arg1 == "unlock" then
				PowerBarsDB.Options.PALADIN.Layout = 5;
				PowerBarsDB.PowSet.Unlocked = 1;
				local Resource = PowerBars.Base.Resource.Base;
				for i=1,3 do
					Resource.Frame[i]:ClearAllPoints();
				end 
			end
			if PowerBarsDB.PowSet.Unlocked == 1 then
				for i=1,#CurrFrame do
					if i == 1 then
						CurrFrame[i].isMovable = false;
						CurrFrame[i].isRescale = false;
					elseif i > 2 then
						CurrFrame[i].isMovable = true;
						CurrFrame[i].isRescale = true;
					end
				end
			else
				for i=1,#CurrFrame do
					if i == 1 then
						CurrFrame[i].isMovable = true;
						CurrFrame[i].isRescale = true;
					elseif i > 2 then
						CurrFrame[i].isMovable = false;
						CurrFrame[i].isRescale = false;
					end
				end
			end
			if PowerBars.isUnlocked then
				PowerBars:doLock();
				PowerBars:doUnlock()
			end
		end
	
		PallyLayout.Items = {
			{ text = "Horizontal (default)", arg1 = "Horiz1" },
			{ text = "Horizontal (inverse)", arg1 = "Horiz2" },
			{ text = "Vertical", arg1 = "Vertic1" },
			{ text = "Vertical (inverse)", arg1 = "Vertic2" },
			{ text = "Unlocked", arg1 = "unlock" },
		};
	
		function PallyLayout.Initialize(self, level)
			local items = PallyLayout.Items;
			local info = UIDropDownMenu_CreateInfo();
			for i=1,#(PallyLayout.Items) do
				info = UIDropDownMenu_CreateInfo();
				info.text = items[i].text;
				info.arg1 = items[i].arg1;
				info.func = PallyLayout.SelectLayout;
				UIDropDownMenu_AddButton(info, level);
			end
		end
		
		UIDropDownMenu_Initialize(PallyLayout, PallyLayout.Initialize)
		UIDropDownMenu_SetWidth(PallyLayout, 150);
		UIDropDownMenu_SetButtonWidth(PallyLayout, 200);
		if not PowerBarsDB.Options.PALADIN then
			UIDropDownMenu_SetSelectedID(PallyLayout, 1);
		else
			UIDropDownMenu_SetSelectedID(PallyLayout, PowerBarsDB.Options.PALADIN.Layout);
		end
		UIDropDownMenu_JustifyText(PallyLayout, "LEFT");
	
		if Class ~= "PALADIN" then
			UIDropDownMenu_DisableDropDown(PallyLayout);
			PallyOptions:Hide();
		else
			UIDropDownMenu_EnableDropDown(PallyLayout);
			PallyOptions:Show();
		end
	end
	
	-- | Warlock |
	if Class == "WARLOCK" then
		PowerBars.Class.WARLOCK = CreateFrame("FRAME", nil, PowerBars.Class);
		local WarlockOptions = PowerBars.Class.WARLOCK;
		WarlockOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		WarlockOptions:SetSize(375, 100);
		WarlockOptions.Title = PowerBars:CreateFont(WarlockOptions, "Warlocks", "DEFAULT", 18, 0, 0, "LEFT", {148,130,201,1});
		WarlockOptions.Title:SetPoint("TOP", WarlockOptions, "LEFT");
		WarlockOptions.Check = PowerBars.Check;
		WarlockOptions.Check.Soulburn = PowerBars:CreateCheck(WarlockOptions, "	PBChecks_Soulburn", "Show Soulburn image");
		local Soulburn = WarlockOptions.Check.Soulburn;
		Soulburn:ClearAllPoints();
	
		if not PowerBars.DropMenus.WarlockLayout then
			PowerBars.DropMenus.WARLOCK.Layout = CreateFrame("FRAME", "PB_WarlockLayout", WarlockOptions, "UIDropDownMenuTemplate");
		end
	
		local WarlockLayout = PowerBars.DropMenus.WARLOCK.Layout;
		WarlockLayout:ClearAllPoints();
		WarlockLayout:SetPoint("TOPLEFT", WarlockOptions.Title, "BOTTOMLEFT", 0, -10);
		WarlockLayout:Show();
		Soulburn:SetPoint("LEFT", WarlockLayout, "RIGHT", 10, 0);
		Soulburn.tooltip = "Toggles whether or not a special image is shown in place of the usual Soulshard image for Soulburn.";
	
		Soulburn:SetScript("OnClick",
			function(self)
				local IsCheck = self:GetChecked();
				if IsCheck == 1 then
					PowerBarsDB.Options.WARLOCK.Soulburn = 1;
					ChangePowWarlock();
				else
					PowerBarsDB.Options.WARLOCK.Soulburn = 0;
					ChangePowWarlock();
				end
			end);
	
		if Class == "WARLOCK" then
			Soulburn:Enable();
			if PowerBarsDB.Options.WARLOCK.Soulburn == 1 then
				Soulburn:SetChecked(true);
			else
				Soulburn:SetChecked(false);
			end
		else
			Soulburn:Disable();
		end
		
		function WarlockLayout.SelectLayout(self, arg1)
			local Frame = PowerBarsDB.PowSet.Frame;
			local Settings = PowerBars.Defaults.WARLOCK.Settings[1];
			UIDropDownMenu_SetSelectedID(WarlockLayout, self:GetID());
			if arg1 == "Horiz1" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.WARLOCK.Layout = 1;
				Frame[1].Width = Settings.WIDTH;
				Frame[1].Height = Settings.HEIGHT;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "TOPLEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "LEFT";
						Settings.RelativePoint = "RIGHT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], Settings.RelativePoint);
					end
				end
			elseif arg1 == "Horiz2" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.WARLOCK.Layout = 2;
				Frame[1].Width = Settings.WIDTH;
				Frame[1].Height = Settings.HEIGHT;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "TOPRIGHT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "RIGHT";
						Settings.RelativePoint = "LEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], 	Settings.RelativePoint);
					end
				end
			elseif arg1 == "Vertic1" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.WARLOCK.Layout = 3;
				Frame[1].Width = 128;
				Frame[1].Height = 192;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "TOPLEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "TOP";
						Settings.RelativePoint = "BOTTOM";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], Settings.RelativePoint);
					end
				end
			elseif arg1 == "Vertic2" then
				PowerBarsDB.PowSet.Unlocked = 0;
				PowerBars.Check.IndivUnlock:SetChecked(false);
				PowerBarsDB.Options.WARLOCK.Layout = 4;
				Frame[1].Width = 128;
				Frame[1].Height = 192;
				PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
				for i=1,3 do
					local Resource = PowerBars.Base.Resource.Base;
					local Settings = Resource.Frame[i].Settings;
					Resource.Frame[i]:ClearAllPoints();
					Settings.Scale = 1;
					Resource.Frame[i]:SetScale(Settings.Scale);
					Settings.X_Ofs = 0;
					Settings.Y_Ofs = 0;
					if i == 1 then
						Settings.Point = "BOTTOMLEFT";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource);
					elseif i > 1 then
						Settings.Point = "BOTTOM";
						Settings.RelativePoint = "TOP";
						Resource.Frame[i]:SetPoint(Settings.Point, Resource.Frame[i-1], Settings.RelativePoint);
					end
				end
			elseif arg1 == "unlock" then
				PowerBarsDB.Options.WARLOCK.Layout = 5;
				PowerBarsDB.PowSet.Unlocked = 1;
				local Resource = PowerBars.Base.Resource.Base;
				for i=1,3 do
					Resource.Frame[i]:ClearAllPoints();
				end 
			end
			if PowerBarsDB.PowSet.Unlocked == 1 then
				for i=1,#CurrFrame do
					if i == 1 then
						CurrFrame[i].isMovable = false;
						CurrFrame[i].isRescale = false;
					elseif i > 2 then
						CurrFrame[i].isMovable = true;
						CurrFrame[i].isRescale = true;
					end
				end
			else
				for i=1,#CurrFrame do
					if i == 1 then
						CurrFrame[i].isMovable = true;
						CurrFrame[i].isRescale = true;
					elseif i > 2 then
						CurrFrame[i].isMovable = false;
						CurrFrame[i].isRescale = false;
					end
				end
			end
			if PowerBars.isUnlocked then
				PowerBars:doLock();
				PowerBars:doUnlock()
			end
		end
	
		WarlockLayout.Items = {
			{ text = "Horizontal (default)", arg1 = "Horiz1" },
			{ text = "Horizontal (inverse)", arg1 = "Horiz2" },
			{ text = "Vertical", arg1 = "Vertic1" },
			{ text = "Vertical (inverse)", arg1 = "Vertic2" },
			{ text = "Unlocked", arg1 = "unlock" },
		};
		
		function WarlockLayout.Initialize(self, level)
			local items = WarlockLayout.Items;
			local info = UIDropDownMenu_CreateInfo();
			for i=1,#(WarlockLayout.Items) do
				info = UIDropDownMenu_CreateInfo();
				info.text = items[i].text;
				info.arg1 = items[i].arg1;
				info.func = WarlockLayout.SelectLayout;
				UIDropDownMenu_AddButton(info, level);
			end
		end
	
		UIDropDownMenu_Initialize(WarlockLayout, WarlockLayout.Initialize)
		UIDropDownMenu_SetWidth(WarlockLayout, 150);
		UIDropDownMenu_SetButtonWidth(WarlockLayout, 200);
		if not PowerBarsDB.Options.WARLOCK then
			UIDropDownMenu_SetSelectedID(WarlockLayout, 1);
		else
			UIDropDownMenu_SetSelectedID(WarlockLayout, PowerBarsDB.Options.WARLOCK.Layout);
		end
		UIDropDownMenu_JustifyText(WarlockLayout, "LEFT");
	
		if Class ~= "WARLOCK" then
			UIDropDownMenu_DisableDropDown(WarlockLayout);
			WarlockOptions:Hide();
		else
			UIDropDownMenu_EnableDropDown(WarlockLayout);
			WarlockOptions:Show();
		end
	end
	
	-- | Druid |
	if Class == "DRUID" then
		PowerBars.Class.DRUID = CreateFrame("FRAME", nil, PowerBars.Class);
		local DruidOptions = PowerBars.Class.DRUID;
		DruidOptions.Balance = CreateFrame("FRAME", nil, DruidOptions);
		local BalanceOptions = DruidOptions.Balance;
		DruidOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		DruidOptions:SetSize(375, 100);
		DruidOptions.Title = PowerBars:CreateFont(DruidOptions, "Druids", "DEFAULT", 18, 0, 0, "LEFT", {255,125,10});
		DruidOptions.Title:SetPoint("TOP", DruidOptions, "LEFT");
		BalanceOptions:SetPoint("TOPLEFT", DruidOptions.Title, "BOTTOMLEFT");
		DruidOptions.Subtitles = {};
		BalanceOptions.Subtitles = DruidOptions.Subtitles;
		BalanceOptions.Subtitles.Spec = PowerBars:CreateFont(BalanceOptions, "(Balance)", "DEFAULT", 14, 0, 0, "LEFT", {255,255,255,1});
		BalanceOptions.Subtitles.Spec:ClearAllPoints();
		BalanceOptions.Subtitles.Spec:SetPoint("TOPLEFT", DruidOptions.Title, "BOTTOMLEFT");
		BalanceOptions.Check = PowerBars.Check;
		BalanceOptions.Check.vTextShow = PowerBars:CreateCheck(BalanceOptions, "	PBChecks_vTextShow", "Show 'Foresee Energy' Text");
		local vTextShow = BalanceOptions.Check.vTextShow;
		vTextShow:ClearAllPoints();
		vTextShow.tooltip = "|cFF00FF00Checked:|cFFFFFFFF Shows predicted energy amounts.\n|cFFFF0000Unchecked:|cFFFFFFFF Does now show predicted energy amounts.\n|cFFFFFF00Original concept by:|cFF4169E1 Copialinex";
		vTextShow:SetPoint("TOPLEFT", DruidOptions.Subtitles.Spec, "BOTTOMLEFT");
	
		BalanceOptions.Check.vWarnShow = PowerBars:CreateCheck(BalanceOptions, "PBChecks_vWarnShow", "Show 'Foresee Energy' Warning");
		local vWarnShow = PowerBars.Check.vWarnShow;
		vWarnShow:ClearAllPoints();
		vWarnShow.tooltip = "|cFF00FF00Checked:|cFFFFFFFF Shows the 'eclipse' image early for lunar and solar eclipse if you're about to hit 100.\n|cFFFF0000Unchecked:|cFFFFFFFF Does now show the 'eclipse' image early for lunar and solar eclipse if you're about to 100.\n|cFFFFFF00Original concept by:|cFF4169E1 Copialinex";
		vWarnShow:SetPoint("TOPLEFT", vTextShow, "BOTTOMLEFT");
	
		BalanceOptions.Check.DruidUnlock = PowerBars:CreateCheck(BalanceOptions, "PBChecks_DruidUnlock", "Unlock Elements");
		local DruidUnlock = BalanceOptions.Check.DruidUnlock;
		DruidUnlock:ClearAllPoints();
		DruidUnlock.tooltip = "When checked, various elements of the Eclipse bar frame are unlocked to be moved or resized individually. You still need to use the unlock interface (/powb unlock) to move or resize them. Elements that can be moved or resized while unlocked include: The 'bar' that the arrow travels along, Eclipse display text and 'foresee' Eclipse display text.";
		DruidUnlock:SetPoint("TOPLEFT", vWarnShow, "BOTTOMLEFT");
		BalanceOptions.Check.isVertical = PowerBars:CreateCheck(BalanceOptions, "PBChecks_isVertical", "Vertical Eclipse Frame");
		local isVertical = BalanceOptions.Check.isVertical;
		isVertical:ClearAllPoints();
		isVertical.tooltip = "|cFF00FF00Checked:|cFFFFFFFF Sets the Eclipse frame to a vertical display.\n|cFFFF0000Unchecked:|cFFFFFFFF Sets the Eclipse frame to its default horizontal display.";
		isVertical:SetPoint("TOPLEFT", DruidUnlock, "BOTTOMLEFT");
	
		BalanceOptions.Sliders = PowerBars.Sliders;
		BalanceOptions.Sliders.Druid = {};
		-- Shapeshift Opacity Slider
		local FormAlpha = 1;
		if Class == "DRUID" then
			FormAlpha = PowerBarsDB.Options.DRUID.FormAlpha;
		end
		BalanceOptions.Sliders.Druid.FormOpacity = PowerBars:CreateSlider(BalanceOptions, "PBSliders_Druid_FormOpacity", "Opacity while Shapeshifted: "..(math.floor(FormAlpha*100)), 0, 1, "1");
		local FormOpacity = PowerBars.Sliders.Druid.FormOpacity;
		FormOpacity:SetValueStep(0.01);
		FormOpacity:SetPoint("TOPLEFT", isVertical, "BOTTOMLEFT", 0, -15);
		FormOpacity:SetValue(FormAlpha);
		FormOpacity.isDragging = 0;
		
		vTextShow:SetScript("OnClick",
			function(self)
				if self:GetChecked() == 1 then
					PowerBarsDB.Options.DRUID.vTextShow = 1;
					PowerBars.Eclipse.vText.Frame:Show();
				else
					PowerBarsDB.Options.DRUID.vTextShow = 0;
					PowerBars.Eclipse.vText.Frame:Hide();
				end
			end);
	
		vWarnShow:SetScript("OnClick",
			function(self)
				if self:GetChecked() == 1 then
					PowerBarsDB.Options.DRUID.vWarnShow = 1;
					PowerBars:ChangePowDruid();
				else
					PowerBarsDB.Options.DRUID.vWarnShow = 0;
					PowerBars:ChangePowDruid();
				end
			end);
	
		DruidUnlock:SetScript("OnClick", doDruidUnlock);
		
		isVertical:SetScript("OnClick",
			function(self)
				if self:GetChecked() == 1 then
					PowerBarsDB.Options.DRUID.isVert = 1;
					PowerBars:SetEclipseVert(1);
				else
					PowerBarsDB.Options.DRUID.isVert = 0;
					PowerBars:SetEclipseVert(0);
				end
			end);
	
		FormOpacity:SetScript("OnMouseDown",
			function(self)
				self.isDragging = 1;
			end);
		
		FormOpacity:SetScript("OnUpdate",
			function(self)
				if self.isDragging == 1 then
					_G[self:GetName() .. 'Text']:SetText("Opacity while Shapeshifted: "..	(math.floor((self:GetValue()*100)+0.5)));
					if Class == "DRUID" and GetPrimaryTalentTree() == 1 then
						Resource.ResTexture:SetAlpha(self:GetValue());
						PowerBars.Eclipse.Arrow.Texture:SetAlpha(self:GetValue());
						PowerBars.Eclipse.Text.Font:SetAlpha(self:GetValue());
						PowerBars.Eclipse.vText.Font:SetAlpha(self:GetValue());
					end
				end
			end); 
		
		FormOpacity:SetScript("OnMouseUp",
			function(self)
				self.isDragging = 0;
				_G[self:GetName() .. 'Text']:SetText("Opacity while Shapeshifted: "..(math.floor(self:GetValue()*100)));
				PowerBarsDB.Options.DRUID.FormAlpha = self:GetValue();
				local currForm = GetShapeshiftFormID()
				if not currForm or currForm == 31 then
					PowerBars:AdjustFormAlpha(1);
				else
					PowerBars:AdjustFormAlpha(0);
				end
			end);
	
		if Class == "DRUID" then
			vTextShow:Enable();
			vWarnShow:Enable();
			FormOpacity:Enable();
			DruidUnlock:Enable();
			isVertical:Enable();
			DruidOptions:Show();
			if PowerBarsDB.Options.DRUID.vTextShow == 1 then
				vTextShow:SetChecked(true);
			else
				vTextShow:SetChecked(false);
			end
			if PowerBarsDB.Options.DRUID.vWarnShow == 1 then
				vWarnShow:SetChecked(true);
			else
				vWarnShow:SetChecked(false);
			end
			if PowerBarsDB.Options.DRUID.Unlocked == 1 then
				DruidUnlock:SetChecked(true);
			else
				DruidUnlock:SetChecked(false);
			end
			if PowerBarsDB.Options.DRUID.isVert == 1 then
				isVertical:SetChecked(true);
			else
				isVertical:SetChecked(false);
			end
		else
			FormOpacity:Disable();
			vTextShow:Disable();
			vWarnShow:Disable();
			DruidUnlock:Disable();
			isVertical:Disable();
			DruidOptions:Hide();
		end
	end	
	-- | Warrior |
	if Class == "WARRIOR" then
		PowerBars.Class.WARRIOR = CreateFrame("FRAME", nil, PowerBars.Class);
		local WarOptions = PowerBars.Class.WARRIOR;
		WarOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		WarOptions:SetSize(375, 100);
		WarOptions.Title = PowerBars:CreateFont(WarOptions, "Warriors", "DEFAULT", 18, 0, 0, "LEFT", {199,156,110,1});
		WarOptions.Title:SetPoint("TOP", WarOptions, "LEFT");
	
		if Class ~= "WARRIOR" then
			WarOptions:Hide();
		else
			WarOptions:Show();
		end
	end
	
	-- | Rogue |
	if Class == "ROGUE" then
		PowerBars.Class.ROGUE = CreateFrame("FRAME", nil, PowerBars.Class);
		local RogueOptions = PowerBars.Class.ROGUE;
		RogueOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		RogueOptions:SetSize(375, 100);
		RogueOptions.Title = PowerBars:CreateFont(RogueOptions, "Rogues", "DEFAULT", 18, 0, 0, "LEFT", {255,245,105,1});
		RogueOptions.Title:SetPoint("TOP", RogueOptions, "LEFT");
	
		if Class ~= "ROGUE" then
			RogueOptions:Hide();
		else
			RogueOptions:Show();
		end
	end
	
	-- | Shaman |
	if Class == "SHAMAN" then
		PowerBars.Class.SHAMAN = CreateFrame("FRAME", nil, PowerBars.Class);
		local ShamOptions = PowerBars.Class.SHAMAN;
		ShamOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		ShamOptions:SetSize(375, 100);
		ShamOptions.Title = PowerBars:CreateFont(ShamOptions, "Shaman", "DEFAULT", 18, 	0, 0, "LEFT", {0,112,222,1});
		ShamOptions.Title:SetPoint("TOP", ShamOptions, "LEFT");
		
		if Class ~= "SHAMAN" then
			ShamOptions:Hide();
		else
			ShamOptions:Show();
		end
	end
	
	-- | Hunter |
	if Class == "HUNTER" then
		PowerBars.Class.HUNTER = CreateFrame("FRAME", nil, PowerBars.Class);
		local HunterOptions = PowerBars.Class.HUNTER;
		HunterOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		HunterOptions:SetSize(375, 100);
		HunterOptions.Title = PowerBars:CreateFont(HunterOptions, "Hunters", "DEFAULT", 18, 0, 0, "LEFT", {171,212,115,1});
		HunterOptions.Title:SetPoint("TOP", HunterOptions, "LEFT");
	
		if Class ~= "HUNTER" then
			HunterOptions:Hide();
		else
			HunterOptions:Show();
		end
	end
	
	-- | Mage |
	if Class == "MAGE" then
		PowerBars.Class.MAGE = CreateFrame("FRAME", nil, PowerBars.Class);
		local MageOptions = PowerBars.Class.MAGE;
		MageOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		MageOptions:SetSize(375, 100);
		MageOptions.Title = PowerBars:CreateFont(MageOptions, "Mages", "DEFAULT", 18, 0, 0, "LEFT", {105,204,240,1});
		MageOptions.Title:SetPoint("TOP", MageOptions, "LEFT");
	
		if Class ~= "MAGE" then
			MageOptions:Hide();
		else
			MageOptions:Show();
		end
	end
	
	-- | Priest |
	if Class == "PRIEST" then
		PowerBars.Class.PRIEST = CreateFrame("FRAME", nil, PowerBars.Class);
		local PriestOptions = PowerBars.Class.PRIEST;
		PriestOptions:SetPoint("TOP", PowerBars.Class, "TOP", 0, -10);
		PriestOptions:SetSize(375, 100);
		PriestOptions.Title = PowerBars:CreateFont(PriestOptions, "Priests", "DEFAULT", 18, 0, 0, "LEFT", {255,255,255,1});
		PriestOptions.Title:SetPoint("TOP", PriestOptions, "LEFT");
	
		if Class ~= "PRIEST" then
			PriestOptions:Hide();
		else
			PriestOptions:Show();
		end
	end
	
	-- *** CUSTOM PANEL ***
	local Custom = PowerBars.Custom;
	Custom.Check = PowerBars.Check;
	Custom.Title = PowerBars:CreateFont(Custom, "Custom Images", "DEFAULT", 18);
	Custom.Title:ClearAllPoints();
	Custom.Title:SetPoint("LEFT", Custom, "TOPLEFT", 15, -20);
	Custom.SubTitles = {};
	-- | CUSTOM RESOURCE |
	Custom.SubTitles.Resource = PowerBars:CreateFont(Custom, "Resources", "DEFAULT", 16);
	Custom.SubTitles.Resource:ClearAllPoints();
	Custom.SubTitles.Resource:SetPoint("TOPLEFT", Custom.Title, "BOTTOMLEFT");
	
	Custom.Check.DefaultIMG = PowerBars:CreateCheck(Custom, "PB_DefaultIMG", "Use Default Image", "TOPLEFT", Custom.SubTitles.Resource, "BOTTOMLEFT");
	local DefaultIMG = Custom.Check.DefaultIMG;
	DefaultIMG.tooltip = "Unchecked means you will use the image in the editbox below. Checked means you will use the default images.";
	
	Custom.Edits = PowerBars.Edits;
	Custom.Edits.SaveResIMG = PowerBars:CreateEdit(Custom, "PBEdits_SaveResIMG", PowerBarsDB.PowSet.SavedResIMG, 150);
	local SaveResIMG = Custom.Edits.SaveResIMG;
	SaveResIMG:SetPoint("TOPLEFT", DefaultIMG, "BOTTOMLEFT", 0, -10);
	SaveResIMG:SetText(PowerBarsDB.PowSet.SavedResIMG);
	SaveResIMG:SetText("TEST");
	SaveResIMG:ClearFocus();
	
	Custom.AspectFrame = CreateFrame("FRAME", nil, Custom);
	Custom.AspectFrame:SetPoint("TOPLEFT", SaveResIMG, "BOTTOMLEFT", 0, -10);
	Custom.AspectFrame:SetSize(375, 150);
	Custom.SubTitles.AspectRes = PowerBars:CreateFont(Custom.AspectFrame, "Aspect Ratio", "DEFAULT", 14);
	Custom.SubTitles.AspectRes:SetPoint("TOPLEFT", Custom.AspectFrame, "TOPLEFT", 0, -10);
	
	Custom.Check.DefAspect = PowerBars:CreateCheck(Custom.AspectFrame, "PB_DefAspect", "Use Default Aspect Ratio", "TOPLEFT", Custom.SubTitles.AspectRes, "BOTTOMLEFT");
	
	Custom.SubTitles.AspectResBase = PowerBars:CreateFont(Custom.AspectFrame, "Base Frame (WxH)");
	Custom.SubTitles.AspectResBase:SetPoint("TOPLEFT", Custom.Check.DefAspect , "BOTTOMLEFT", 0, -10);
	Custom.Edits.Aspect = {};
	Custom.Edits.Aspect.Resource = {};
	Custom.Edits.Aspect.Resource.Base = {};
	local AResource = Custom.Edits.Aspect.Resource;
	AResource.Base.Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_AspectResourceBaseW", PowerBarsDB.PowSet.Frame[1].Width, "def", "def", "notMulti", "isNum");
	AResource.Base.Width:SetPoint("TOPLEFT", Custom.SubTitles.AspectResBase, "BOTTOMLEFT", 0, -2);
	Custom.SubTitles.Divider = {};
	Custom.SubTitles.Divider[1] = PowerBars:CreateFont(Custom.AspectFrame, "x");
	Custom.SubTitles.Divider[1]:ClearAllPoints();
	Custom.SubTitles.Divider[1]:SetPoint("LEFT", AResource.Base.Width, "RIGHT", 1, 0);
	AResource.Base.Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_AspectResourceBaseH", PowerBarsDB.PowSet.Frame[1].Height, "def", "def", "notMulti", "isNum");
	AResource.Base.Height:SetPoint("LEFT", Custom.SubTitles.Divider[1], "RIGHT", 5, 0);

	
	if Class == "DRUID" then
		Custom.SubTitles.Eclipse = PowerBars:CreateFont(Custom.AspectFrame, "Arrow Frame");
		Custom.SubTitles.Eclipse:ClearAllPoints();
		Custom.SubTitles.Eclipse:SetPoint("LEFT", Custom.SubTitles.AspectResBase, "RIGHT", 30, 0);
		AResource.Arrow = {};
		AResource.Arrow.Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_ArrowWidth", PowerBarsDB.PowSet.Frame[3].Width, "def", "def", "notMulti", "isNum");
		AResource.Arrow.Width:SetPoint("LEFT", AResource.Base.Height, "RIGHT", 20, 0)
		Custom.SubTitles.Divider[2] = PowerBars:CreateFont(Custom.AspectFrame, "x");
		Custom.SubTitles.Divider[2]:ClearAllPoints();
		Custom.SubTitles.Divider[2]:SetPoint("LEFT", AResource.Arrow.Width, "RIGHT", 1, 0);
		AResource.Arrow.Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_ArrowHeight", PowerBarsDB.PowSet.Frame[3].Height, "def", "def", "notMulti", "isNum");
		AResource.Arrow.Height:SetPoint("LEFT", Custom.SubTitles.Divider[2], "RIGHT", 5, 0);
		AResource.SetAspect = PowerBars:CreateButton(Custom.AspectFrame, "PBButton_SetAspect", "Set Aspect Ratios", 0, 0, 150);
		AResource.SetAspect:ClearAllPoints();
		AResource.SetAspect:SetPoint("TOP", AResource.Base.Height, "BOTTOMRIGHT");
	elseif Class == "DEATHKNIGHT" then
		Custom.SubTitles.Blood = PowerBars:CreateFont(Custom.AspectFrame, "Blood");
		Custom.SubTitles.Blood:ClearAllPoints();
		Custom.SubTitles.Blood:SetPoint("LEFT", Custom.SubTitles.AspectResBase, "RIGHT", 40, 0);
		Custom.SubTitles.Unholy = PowerBars:CreateFont(Custom.AspectFrame, "Unholy");
		Custom.SubTitles.Unholy:ClearAllPoints();
		Custom.SubTitles.Unholy:SetPoint("LEFT", Custom.SubTitles.Blood, "RIGHT", 60, 0);
		Custom.SubTitles.Frost = PowerBars:CreateFont(Custom.AspectFrame, "Frost");
		Custom.SubTitles.Frost:ClearAllPoints();
		Custom.SubTitles.Frost:SetPoint("TOP", AResource.Base.Height, "BOTTOMRIGHT");
		Custom.SubTitles.Death = PowerBars:CreateFont(Custom.AspectFrame, "Death");
		Custom.SubTitles.Death:ClearAllPoints();
		Custom.SubTitles.Death:SetPoint("LEFT", Custom.SubTitles.Frost, "RIGHT", 75, 0);
		AResource.Blood = {};
		AResource.Blood.Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_BloodWidth", PowerBarsDB.PowSet.Frame[3].Width, 35, "def", "notMulti", "isNum");
		AResource.Blood.Width:SetPoint("LEFT", AResource.Base.Height, "RIGHT", 20, 0);
		AResource.Blood.Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_BloodHeight", PowerBarsDB.PowSet.Frame[3].Height, 35, "def", "notMulti", "isNum");
		Custom.SubTitles.Divider[2] = PowerBars:CreateFont(Custom.AspectFrame, "x");
		Custom.SubTitles.Divider[2]:ClearAllPoints();
		Custom.SubTitles.Divider[2]:SetPoint("LEFT", AResource.Blood.Width, "RIGHT", 1, 0);
		AResource.Blood.Height:SetPoint("LEFT", Custom.SubTitles.Divider[2], "RIGHT", 5, 0);
		AResource.Unholy = {};
		AResource.Unholy.Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_UnholyWidth", PowerBarsDB.PowSet.Frame[5].Width, 35, "def", "notMulti", "isNum");
		AResource.Unholy.Width:SetPoint("LEFT", AResource.Blood.Height, "RIGHT", 20, 0);
		AResource.Unholy.Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_UnholyHeight", PowerBarsDB.PowSet.Frame[5].Height, 35, "def", "notMulti", "isNum");
		Custom.SubTitles.Divider[3] = PowerBars:CreateFont(Custom.AspectFrame, "x");
		Custom.SubTitles.Divider[3]:ClearAllPoints();
		Custom.SubTitles.Divider[3]:SetPoint("LEFT", AResource.Unholy.Width, "RIGHT", 1, 0);
		AResource.Unholy.Height:SetPoint("LEFT", Custom.SubTitles.Divider[3], "RIGHT", 5, 0);
		AResource.Frost = {};
		AResource.Frost.Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_FrostWidth", PowerBarsDB.PowSet.Frame[7].Width, 35, "def", "notMulti", "isNum");
		AResource.Frost.Width:SetPoint("TOPLEFT", Custom.SubTitles.Frost, "BOTTOMLEFT", -25, -2);
		AResource.Frost.Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_FrostHeight", PowerBarsDB.PowSet.Frame[7].Height, 35, "def", "notMulti", "isNum");
		Custom.SubTitles.Divider[4] = PowerBars:CreateFont(Custom.AspectFrame, "x");
		Custom.SubTitles.Divider[4]:ClearAllPoints();
		Custom.SubTitles.Divider[4]:SetPoint("LEFT", AResource.Frost.Width, "RIGHT", 1, 0);
		AResource.Frost.Height:SetPoint("LEFT", Custom.SubTitles.Divider[4], "RIGHT", 5, 0);
		AResource.Death = {};
		AResource.Death.Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_DeathWidth", PowerBarsDB.PowSet.Frame[9].Width, 35, "def", "notMulti", "isNum");
		AResource.Death.Width:SetPoint("LEFT", AResource.Frost.Height, "RIGHT", 20, 0);
		AResource.Death.Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_DeathHeight", PowerBarsDB.PowSet.Frame[9].Height, 35, "def", "notMulti", "isNum");
		Custom.SubTitles.Divider[5] = PowerBars:CreateFont(Custom.AspectFrame, "x");
		Custom.SubTitles.Divider[5]:ClearAllPoints();
		Custom.SubTitles.Divider[5]:SetPoint("LEFT", AResource.Death.Width, "RIGHT", 1, 0);
		AResource.Death.Height:SetPoint("LEFT", Custom.SubTitles.Divider[5], "RIGHT", 5, 0);
		AResource.SetAspect = PowerBars:CreateButton(Custom.AspectFrame, "PBButton_SetAspect", "Set Aspect Ratios", 0, 0, 150);
		AResource.SetAspect:ClearAllPoints();
		AResource.SetAspect:SetPoint("TOP", AResource.Frost.Height, "BOTTOMRIGHT");
	elseif Class == "PALADIN" or Class == "WARLOCK" then
		Custom.SubTitles.Frame = {};
		for i=1,3 do
			if Class == "PALADIN" then
				Custom.SubTitles.Frame[i] = PowerBars:CreateFont(Custom.AspectFrame, "HPW "..i);
			else
				Custom.SubTitles.Frame[i] = PowerBars:CreateFont(Custom.AspectFrame, "Shard "..i);
			end
			Custom.SubTitles.Frame[i]:ClearAllPoints();
		end
		Custom.SubTitles.Frame[1]:SetPoint("TOPLEFT", AResource.Base.Width, "BOTTOMLEFT", 20, 0);
		Custom.SubTitles.Frame[2]:SetPoint("LEFT", Custom.SubTitles.Frame[1], "RIGHT", 60, 0);
		if Class == "PALADIN" then
			Custom.SubTitles.Frame[3]:SetPoint("LEFT", Custom.SubTitles.Frame[2], "RIGHT", 60, 0);
		else
			Custom.SubTitles.Frame[3]:SetPoint("LEFT", Custom.SubTitles.Frame[2], "RIGHT", 50, 0);
		end
		AResource.Frame = {};
		for i=1,3 do
			AResource.Frame[i] = {};
			AResource.Frame[i].Width = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_Frame"..i.."Width", PowerBarsDB.PowSet.Frame[i+2].Width, 35, "def", "notMulti", "isNum");
			if i == 1 then
				AResource.Frame[i].Width:SetPoint("TOPLEFT", Custom.SubTitles.Frame[1], "BOTTOMLEFT", -20, 0);
			else
				AResource.Frame[i].Width:SetPoint("LEFT", AResource.Frame[i-1].Height, "RIGHT", 20, 0);
			end
			AResource.Frame[i].Height = PowerBars:CreateEdit(Custom.AspectFrame, "PBEdits_Frame"..i.."Height", PowerBarsDB.PowSet.Frame[i+2].Height, 35, "def", "notMulti", "isNum");
			Custom.SubTitles.Divider[i+1] = PowerBars:CreateFont(Custom.AspectFrame, "x");
			Custom.SubTitles.Divider[i+1]:ClearAllPoints();
			Custom.SubTitles.Divider[i+1]:SetPoint("LEFT", AResource.Frame[i].Width, "RIGHT", 1, 0);
			AResource.Frame[i].Height:SetPoint("LEFT", Custom.SubTitles.Divider[i+1], "RIGHT", 5, 0);
		end
		AResource.SetAspect = PowerBars:CreateButton(Custom.AspectFrame, "PBButton_SetAspect", "Set Aspect Ratios", 0, 0, 150);
		AResource.SetAspect:ClearAllPoints();
		AResource.SetAspect:SetPoint("TOPLEFT", AResource.Frame[1].Height, "BOTTOM");
	else
		AResource.SetAspect = CreateFrame("BUTTON", "PBButton_SetAspect", Custom.AspectFrame);
	end
	
	AResource.SetAspect:SetScript("OnClick",
		function()
			local Frame = PowerBarsDB.PowSet.Frame;
			if AResource.Base.Width:GetText() ~= "" then
				Frame[1].Width = tonumber(AResource.Base.Width:GetText());
			else
				Frame[1].Width = Frame[1].Width;
			end
			if AResource.Base.Height:GetText() ~= "" then
				Frame[1].Height = tonumber(AResource.Base.Height:GetText());
			else
				Frame[1].Height = Frame[1].Height;
			end
			PowerBars.Base.Resource.Base:SetSize(Frame[1].Width, Frame[1].Height);
			if Class == "PALADIN" or Class == "WARLOCK" then
				local Resource = PowerBars.Base.Resource.Base;
				for i=1,3 do
					if AResource.Frame[i].Width:GetText() ~= "" then
						Frame[i+2].Width = tonumber(AResource.Frame[i].Width:GetText());
					else
						Frame[i+2].Width = 0;
					end
					if AResource.Frame[i].Height:GetText() ~= "" then
						Frame[i+2].Height = tonumber(AResource.Frame[i].Height:GetText());
					else
						Frame[i+2].Height = 0;
					end
					Resource.Frame[i]:SetSize(Frame[i+2].Width, Frame[i+2].Height);
				end
			elseif Class == "DEATHKNIGHT" then
				local Runes = PowerBars.Runes;
				-- Blood
				if AResource.Blood.Width:GetText() ~= "" then
					Frame[3].Width = tonumber(AResource.Blood.Width:GetText());
					Frame[4].Width = tonumber(AResource.Blood.Width:GetText());
				else
					Frame[3].Width = Frame[3].Width;
					Frame[4].Width = Frame[4].Width;
				end
				if AResource.Blood.Height:GetText() ~= "" then
					Frame[3].Height = tonumber(AResource.Blood.Height:GetText());
					Frame[4].Height = tonumber(AResource.Blood.Height:GetText());
				else
					Frame[3].Height = Frame[3].Height;
					Frame[4].Height = Frame[4].Height;
				end
				for i=1,2 do
					Runes[i].Frame:SetSize(Frame[3].Width, Frame[3].Height);
				end
				-- Unholy
				if AResource.Unholy.Width:GetText() ~= "" then
					Frame[5].Width = tonumber(AResource.Unholy.Width:GetText());
					Frame[6].Width = tonumber(AResource.Unholy.Width:GetText());
				else
					Frame[5].Width = Frame[5].Width;
					Frame[6].Width = Frame[6].Width;
				end
				if AResource.Unholy.Height:GetText() ~= "" then
					Frame[5].Height = tonumber(AResource.Unholy.Height:GetText());
					Frame[6].Height = tonumber(AResource.Unholy.Height:GetText());
				else
					Frame[5].Height = Frame[5].Height;
					Frame[6].Height = Frame[6].Height;
				end
				for i=3,2 do
					Runes[i].Frame:SetSize(Frame[5].Width, Frame[5].Height);
				end
				-- Frost
				if AResource.Frost.Width:GetText() ~= "" then
					Frame[7].Width = tonumber(AResource.Frost.Width:GetText());
					Frame[8].Width = tonumber(AResource.Frost.Width:GetText());
				else
					Frame[7].Width = Frame[7].Width;
					Frame[8].Width = Frame[8].Width;
				end
				if AResource.Frost.Height:GetText() ~= "" then
					Frame[7].Height = tonumber(AResource.Frost.Height:GetText());
					Frame[8].Height = tonumber(AResource.Frost.Height:GetText());
				else
					Frame[7].Height = Frame[7];
					Frame[8].Height = Frame[8];
				end
				for i=5,2 do
					Runes[i].Frame:SetSize(Frame[7].Width, Frame[7].Height);
				end
				-- Death
				if AResource.Death.Width:GetText() ~= "" then
					Frame[9].Width = tonumber(AResource.Death.Width:GetText());
				else
					Frame[9].Width = Frame[9].Width;
				end
				if AResource.Death.Height:GetText() ~= "" then
					Frame[9].Height = tonumber(AResource.Death.Height:GetText());
				else
					Frame[9].Height = Frame[9].Height;
				end
			elseif Class == "DRUID" then
				if AResource.Arrow.Width:GetText() ~= "" then
					Frame[3].Width = tonumber(AResource.Arrow.Width:GetText());
				else
					Frame[3].Width = Frame[3].Width;
				end
				if AResource.Arrow.Height:GetText() ~= "" then
					Frame[3].Height = tonumber(AResource.Arrow.Height:GetText());
				else
					Frame[3].Height = Frame[3].Height;
				end
				PowerBars.Eclipse.Arrow.Frame:SetSize(Frame[3].Width, Frame[3].Height);
			end
		end);
	
	Custom.Check.DefAspect:SetScript("OnClick",
		function(self)
			local IsCheck = self:GetChecked();
			if IsCheck == 1 then
				PowerBarsDB.PowSet.Aspect.Resource = 0;
				AResource.SetAspect:Disable();
				local Defaults = PowerBars.Defaults[Class];
				local Frame = PowerBarsDB.PowSet.Frame;
				Frame[1].Width = Defaults.Settings[1].WIDTH;
				Frame[1].Height = Defaults.Settings[1].HEIGHT;
				PowerBars.Base.Resource.Base:SetSize(Defaults.Settings[1].WIDTH, Defaults.Settings[1].HEIGHT);
				if Class == "PALADIN" or Class == "WARLOCK" then
					local Resource = PowerBars.Base.Resource.Base;
					for i=1,3 do
						Resource.Frame[i]:SetSize(Defaults.Settings[i+2].WIDTH, Defaults.Settings[i+2].HEIGHT);
						Frame[i+2].Width = Defaults.Settings[i+2].WIDTH;
						Frame[i+2].Height = Defaults.Settings[i+2].HEIGHT;
					end
				elseif Class == "DEATHKNIGHT" then
					local Runes = PowerBars.Runes;
					for i=1,6 do
						local Rune = Runes[i]
						Rune.Frame:SetSize(Defaults.Settings[i+2].WIDTH, Defaults.Settings[i+2].HEIGHT);
						Frame[i+2].Width = Defaults.Settings[i+2].WIDTH;
						Frame[i+2].Height = Defaults.Settings[i+2].HEIGHT;
					end
				elseif Class == "DRUID" then
					if GetPrimaryTalentTree() == 1 then
						PowerBars.Eclipse.Arrow.Frame:SetSize(Defaults.Settings[3].WIDTH, Defaults.Settings[3].HEIGHT);
						Frame[3].Width = Defaults.Settings[3].WIDTH;
						Frame[3].Height = Defaults.Settings[3].HEIGHT;
					end
				end
			else
				PowerBarsDB.PowSet.Aspect.Resource = 1;
				AResource.SetAspect:Enable();
			end
		end);
	
	-- || Check Box Scripts ||
	DefaultIMG:SetScript("OnClick",
		function(self)
			local IsCheck = self:GetChecked();
			if IsCheck == 1 then
				PowerBarsDB.PowSet.DefaultIMG = 1;
				GetInfo();
				SetClassImg();
			else
				PowerBarsDB.PowSet.DefaultIMG = 0;
				PowerBarsDB.PowSet.PowIMG = PowerBarsDB.PowSet.SavedResIMG;
				SetClassImg();
			end
		end);
		
	-- || Edit Box Scripts ||
	SaveResIMG:SetScript("OnEditFocusLost",
		function(self)
			PowerBarsDB.PowSet.SavedResIMG = self:GetText();
			if PowerBarsDB.PowSet.DefaultIMG == 0 then
				PowerBarsDB.PowSet.PowIMG = self:GetText();
			end
		end);
	SaveResIMG:SetScript("OnEnterPressed",
		function(self)
			PowerBarsDB.PowSet.SavedResIMG = self:GetText();
			if PowerBarsDB.PowSet.DefaultIMG == 0 then
				PowerBarsDB.PowSet.PowIMG = self:GetText();
			end
		end);
	
	if Class == "PALADIN" or Class == "WARLOCK" or Class == "DEATHKNIGHT" or Class == "DRUID" then
		DefaultIMG:Enable();
		Custom.Check.DefAspect:Enable();
		Custom.AspectFrame:Show();
		if PowerBarsDB.PowSet.DefaultIMG == 1 then
			DefaultIMG:SetChecked(true);
		else
			DefaultIMG:SetChecked(false);
		end
		if PowerBarsDB.PowSet.Aspect.Resource == 0 then
			Custom.Check.DefAspect:SetChecked(true);
			AResource.SetAspect:Disable();
		else
			Custom.Check.DefAspect:SetChecked(false);
			AResource.SetAspect:Enable();
		end
	else
		DefaultIMG:Disable();
		Custom.Check.DefAspect:Disable();
		Custom.AspectFrame:Hide();
	end
	
	-- | CUSTOM POWER |
	Custom.SubTitles.Resource = PowerBars:CreateFont(Custom, "Power", "DEFAULT", 16);
	Custom.SubTitles.Resource:ClearAllPoints();
	Custom.SubTitles.Resource:SetPoint("TOPLEFT", Custom.AspectFrame, "BOTTOMLEFT");
end