local IMG =  "Interface\\AddOns\\PowerBars\\imgs\\";
local MsgLog = PowerBars.MsgLog;

function PowerBars:CallError(errorString, showError)
	local showError = showError or "doShow";
	if showError == "doShow" then
		if PowerBarsDB.Options.ErrorConsole == 0 then
			print("|cFFFFFF00PowerBars: |cFFFF0000ERROR: |cFFFFFFFF"..errorString.." | type: '/powb log' for more details.");
		else
			print("|cFFFFFF00PowerBars: |cFFFF0000ERROR: |cFFFFFFFF"..errorString);
		end
	end
	MsgLog:AddLine("|cFFFF0000Error:|cFFFFFFFF "..errorString, "Error");
	if PowerBarsDB.Options.ErrorConsole == 1 then
		PowerBars:CallMsgConsole();
	end
end

function MsgLog:AddLine(lineString, lineType, lineNum)
	local lineNum = lineNum or 0;
	local lineType = lineType or "General";
	local NumLines = #(self.Lines)
	if NumLines == 1000 then
		PowerBars:CollapseTable(self.Lines,1,1000);
		NumLines = #(self.Lines);
	elseif NumLines > 1000 then
		print("Uuuh");	
	end
	self.Lines[(NumLines+1)] = {};
	self.Lines[(NumLines+1)].String = lineString;
	self.Lines[(NumLines+1)].Num = lineNum;
	self.Lines[(NumLines+1)].Type = lineType;
	local fTime = date("%m/%d/%y %H:%M:%S");
	local fTime = string.format("%d %d %d", string.match(fTime, "%d+/%d+/%d+ (%d+):(%d+):(%d+)"));
	local pos1 = string.find(fTime, " ");
	local pos2 = string.find(fTime, " ", pos1+1);
	local hour = string.sub(fTime, 1, (pos1-1));
	local minute = string.sub(fTime, (pos1+1), (pos2-1));
	local sec = string.sub(fTime, (pos2+1));
	hour = tonumber(hour);
	minute = tonumber(minute);
	sec = tonumber(sec);
	if hour > 12 then
		hour = hour - 12;
	end
	local fHour, fMin, fSec = "", "", "";
	if hour < 10 then
		hour = "0" .. hour;
	end
	if minute < 10 then
		minute = "0" .. minute;
	end
	if sec < 10 then
		sec = "0" .. sec;
	end
	fTime = hour .. ":" .. minute .. ":" .. sec;
	self.Lines[(NumLines+1)].fTime = fTime;
	self.Lines[(NumLines+1)].Time = math.floor(GetTime());
	PowerBars:SetLines(#(self.Lines));
end

function PowerBars:SetLines(endAt)
	local NumLines = #(MsgLog.Lines);
	local startFrom = 0;
	local endAt = endAt or NumLines;
	
	for i=1,30 do
		MsgLog.Console.Frame.ScrollFrame.Lines[i]:SetText("");
	end
					
	if NumLines >= 30 and endAt >= 30 then
		startFrom = endAt-30
		endAt = 30;
	end

	local MsgText = "";
	if NumLines >= 1000 then
		MsgLog.Console.Frame.LineBox:SetSize(450,510);
		MsgLog.Console.Frame.LineBox:SetPoint("TOPLEFT", MsgLog.Console.Frame.ScrollFrame, "TOPLEFT", 42, 0);
	elseif NumLines >= 100 then
		MsgLog.Console.Frame.LineBox:SetSize(460,510);
		MsgLog.Console.Frame.LineBox:SetPoint("TOPLEFT", MsgLog.Console.Frame.ScrollFrame, "TOPLEFT", 32, 0);
	elseif NumLines >= 10 then
		MsgLog.Console.Frame.LineBox:SetSize(470,510);
		MsgLog.Console.Frame.LineBox:SetPoint("TOPLEFT", MsgLog.Console.Frame.ScrollFrame, "TOPLEFT", 22, 0);
	else
		MsgLog.Console.Frame.LineBox:SetSize(480,510);
		MsgLog.Console.Frame.LineBox:SetPoint("TOPLEFT", MsgLog.Console.Frame.ScrollFrame, "TOPLEFT", 12, 0);
	end
	for i=1,endAt do
		--local formattedText = (startFrom+i) .. ". " .. MsgLog.Lines[startFrom+i].fTime .. ": ";
		local formattedText = "|cFFFFFFFF";
		if i ~= 1 then
			formattedText = "\n" .. formattedText;
		end
		formattedText = formattedText .. MsgLog.Lines[startFrom+i].fTime .. ": ";
		if MsgLog.Lines[startFrom+i].Num > 0 then
			formattedText = formattedText .. " Line#: " .. MsgLog.Lines[startFrom+i].Num .. ": ";
		end
		formattedText = formattedText .. MsgLog.Lines[startFrom+i].String;
		local currLine = (startFrom+i) .. ". ";
		--currLine = string.format("%4s", currLine)
		MsgLog.Console.Frame.ScrollFrame.Lines[i]:SetText(currLine);
		--[[if i ~= endAt-1 then
			MsgLogText = MsgLogText .. formattedText .. "\n";
		else]]
			MsgText = MsgText .. formattedText;
		--end
	end
	MsgLog.Text = MsgText;
	MsgLog.Console.Frame.LineBox:SetText(MsgLog.Text);
end

function PowerBars:CallMsgConsole()
	if not MsgLog.Console then
		MsgLog.Console = {};
		MsgLog.Console.Frame = CreateFrame("FRAME", nil, UIParent);
		local Frame = MsgLog.Console.Frame;
		Frame:SetFrameStrata("DIALOG");
		Frame:SetToplevel(true);
		Frame:EnableMouse(true);
		Frame:SetSize(600,500);
		Frame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
			tile = true,
			insets = {left = 11, right = 12, top = 12, bottom = 11},
			tileSize = 32,
			edgeSize = 32,	
		});
		Frame:SetPoint("TOP", UIParent, "TOP", 0, -10);
		Frame:SetMovable(true);
		Frame.isMoving = false;
		
		Frame:SetScript("OnMouseDown", 
		function(self, button)
			if button == "LeftButton" and not self.isMoving then
				self:StartMoving();
				self.isMoving = true;
			end
		end);
		Frame:SetScript("OnMouseUp",
		function(self, button)
			if self.isMoving then
				self:StopMovingOrSizing();
				self.isMoving = false;
			end
		end);
		
		Frame:Hide();
		
		MsgLog.Console.Title = PowerBars:CreateFont(Frame, "Power Bars: Message Console", "DEFAULT", 14);
		local Title = MsgLog.Console.Title;
		Title:ClearAllPoints();
		Title:SetPoint("TOP", Frame, "TOP", 0, -15);
		
		MsgLog.Console.Buttons = PowerBars.Buttons;
		MsgLog.Console.Buttons.MsgLog = {};
		MsgLog.Console.Buttons.MsgLog.Close = PowerBars:CreateButton(Frame, "PBMsgLogClose", "Close");
		local Close = MsgLog.Console.Buttons.MsgLog.Close;
		Close:ClearAllPoints();
		Close:SetPoint("TOP", Title, "BOTTOM", 0, -5);
		Close:SetScript("OnClick",
			function()
				Frame:Hide();
			end);
			
		Frame.LinesFrame = CreateFrame("FRAME", nil, Frame);
		local LinesFrame = Frame.LinesFrame;
		LinesFrame:SetSize(520,400);
		LinesFrame:SetPoint("TOPLEFT", Close, "BOTTOMLEFT", -200, -10);
		LinesFrame:SetBackdrop({
			bgFile = nil,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 5, right = 5, bottom = 5, top = 5},
		});
		LinesFrame.Texture = LinesFrame:CreateTexture(nil, "BACKGROUND");
		LinesFrame.Texture:SetTexture(0,1,0,0);
		LinesFrame.Texture:SetAllPoints(LinesFrame);
		
		Frame.ScrollFrame = CreateFrame("SCROLLFRAME", nil, Frame);
		local ScrollFrame = Frame.ScrollFrame;
		ScrollFrame:SetSize(510,390);
		ScrollFrame:SetPoint("TOPLEFT", LinesFrame, "TOPLEFT", 5, -5);
		ScrollFrame.Texture = ScrollFrame:CreateTexture(nil, "BACKGROUND");
		ScrollFrame.Texture:SetTexture(0,0,0,0.75);
		ScrollFrame.Texture:SetAllPoints(ScrollFrame);
		
		Frame.LineBox = PowerBars:CreateEdit(Frame, "PBEdits_FrameLineBox", nil, 510, 510, "isMulti", "notNum", "notPass", 0, {5,5,15,5}, 1);
		ScrollFrame:SetScrollChild(Frame.LineBox);
		Frame.LineBox:ClearAllPoints();
		Frame.LineBox:SetSize(480,510);
		Frame.LineBox:SetPoint("TOPLEFT", ScrollFrame, "TOPLEFT", 12, 0);

		ScrollFrame:SetScript("OnMouseDown",
			function(self)
				Frame.LineBox:SetFocus();
			end);
		
		--[[
		Frame.LineNum = CreateFrame("FRAME", nil, Frame);
		local LineNum = Frame.LineNum;
		LineNum:SetSize(510, ]]
		
		-- Temporary until the Scroll Edit Box is imped
		ScrollFrame.Lines = {};
		for i=1,30 do
			ScrollFrame.Lines[i] = PowerBars:CreateFont(LinesFrame, "", "DEFAULT", 12, 505, 0, "LEFT", {255,255,255,1});
			ScrollFrame.Lines[i]:ClearAllPoints();
			if i == 1 then
				ScrollFrame.Lines[i]:SetPoint("TOPLEFT", LinesFrame, "TOPLEFT", 5, -19);
			else
				ScrollFrame.Lines[i]:SetPoint("TOP", ScrollFrame.Lines[i-1], "BOTTOM", 0, 0);
			end
		end
	else
	--[[
	local NumLines = #(MsgLog.Lines)
	if NumLines > 0 then
		PowerBars:SetLines(NumLines);
	end]]
	MsgLog.Console.Frame:Show();
	end
end

function PowerBars:StringFormat(formatString, padTo)
	local strLen = string.len(formatString);
	if strLen < padTo then
		formatString = formatString .. string.rep(" ", ((padTo-strLen)+1))
	elseif strLen > padTo then
		formatString = string.sub(formatString, 1, (padTo-3));
		formatString = formatString .. "...";
	end
	return formatString;
end

function PowerBars:CreateCheck(parent, name, displayname, point, relativeTo, relativePoint, x_loc, y_loc)
	local CheckButton = CreateFrame("CheckButton", name, parent, "ChatConfigCheckButtonTemplate");
	local point = point or "TOPLEFT";
	local relativeTo = relativeTo or parent;
	local relativePoint = relativePoint or "TOPLEFT";
	local x_loc = x_loc or 0;
	local y_loc = y_lock or 0;
	CheckButton:SetPoint(point, relativeTo, relativePoint, x_loc, y_loc);
	displayname = "  " .. displayname;
	_G[CheckButton:GetName() .. "Text"]:SetText(displayname);
	return CheckButton;
end


function PowerBars:CreateButton(parent, name, displayname, x_loc, y_loc, setwidth, setheight)
	local setheight = setheight or 25;
	local setwidth = setwidth or 120;
	local x_loc = x_loc or 0;
	local y_loc = y_loc or 0;
	local Button = CreateFrame("Button", name, parent, "UIPanelButtonTemplate");
	Button:SetPoint("TOPLEFT", x_loc, y_loc);
	Button:SetText(displayname);
	Button:SetHeight(setheight);
	Button:SetWidth(setwidth);

	return Button;
end

function PowerBars:CreateFont(parent, fonttext, fonttype, fontsize, fontwidth, fontheight, fontjustify, fontcolor, fontlayer, fontflags)
	local fontjustify = fontjustify or "LEFT";
	local fontwidth = fontwidth or 0;
	local fontheight = fontheight or 0;
	local fontcolor = fontcolor or {255,202,0,1};
	local fontsize = fontsize or 12;
	local fonttype = fonttype or "DEFAULT";
	local fontlayer = fontlayer or "DRAWLAYER";
	local fontstring = parent:CreateFontString(nil, "DRAWLAYER");
	if fonttype == "DEFAULT" then
		if not fontflags then
			fontstring:SetFont("Fonts\\FRIZQT__.TTF", fontsize);
		else
			fontstring:SetFont("Fonts\\FRIZQT__.TTF", fontsize, fontflags);
		end
	else
		if not fontflags then
			fontstring:SetFont(fonttype, fontsize);
		else
			fontstring:SetFont(fonttype, fontsize, fontflags);
		end
	end
	fontstring:SetText(fonttext);
	if fontheight ~= 0 then
		fontstring:SetHeight(fontheight);
	end
	if fontwidth ~= 0 then
		fontstring:SetWidth(fontwidth);
	end
	fontstring:SetJustifyH(fontjustify);
	fontstring:SetTextColor(fontcolor[1]/255,fontcolor[2]/255,fontcolor[3]/255,fontcolor[4]);
	fontstring:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);	
	return fontstring;
end

function PowerBars:CreateSlider(parent, name, title, min_val, max_val, lowtext, hightext, width, height, orientation)
	local min_val = min_val or 0;
	local max_val = max_val or 100;
	local width = width or 300;
	local height = height or 15;
	local lowtext = lowtext or "1";
	local hightext = hightext or "100";
	local orientation = orientation or 'HORIZONTAL';
	local newSlider = CreateFrame("SLIDER", name, parent, "OptionsSliderTemplate");
	newSlider:SetSize(width, height);
	newSlider:SetOrientation(orientation);
	newSlider:SetMinMaxValues(min_val, max_val);
	_G[newSlider:GetName() .. 'Low']:SetText(lowtext);
 	_G[newSlider:GetName() ..  'High']:SetText(hightext);
 	_G[newSlider:GetName() .. 'Text']:SetText(title);
	return newSlider;
end

function PowerBars:CreateEdit(parent, name, text, width, height, multi, numeric, password, maxletters, insets, novis)
	local width = width or 50;
	local height = height or 20;
	if width == "def" then
		width = 50;
	end
	if height == "def" then
		height = 20;
	end
	local multi = multi or 0;
	local numeric = numeric or 0;
	local password = password or 0;
	local maxletters = maxletters or 0;
	local novis = novis or 0;
	local text = text or nil;
	local insets = insets or {0,0,0,0};
	local newEditBox = "";
	if novis == 0 then
		newEditBox = CreateFrame("EDITBOX", name, parent, "INPUTBOXTEMPLATE");
	else
		newEditBox = CreateFrame("EDITBOX", name, parent);
		newEditBox:SetFont("Fonts\\FRIZQT__.TTF", 12);
	end
	if text then
		newEditBox:SetText(text);
	end
	newEditBox:SetSize(width, height);
	if multi == true or multi == "isMulti" then
		newEditBox:SetMultiLine(true);
	end
	if numeric == true or numeric == "isNum" then
		newEditBox:SetNumeric(true);
	end
	if password == true or password == "isPass" then
		newEditBox:SetPassword(true);
	end
	if maxletters > 0 then
		newEditBox:SetMaxLetters(maxletters);
	end
	newEditBox:SetTextInsets(unpack(insets));
	newEditBox:SetScript("OnEscapePressed", 
		function(self)
			self:ClearFocus(); 
		end);
	newEditBox:SetAutoFocus(false);
	return newEditBox;
end